#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import os
import wiredtiger, wttest
from suite_subprocess import suite_subprocess

# test_jsondump.py
# Test dump output from json cursors.
class test_jsondump02(wttest.WiredTigerTestCase, suite_subprocess):

    table_uri1 = 'table:jsondump02a.wt'
    table_uri2 = 'table:jsondump02b.wt'
    table_uri3 = 'table:jsondump02c.wt'
    basename_uri4 = 'jsondump02d.wt'
    table_uri4 = 'table:' + basename_uri4
    table_uri5 = 'table:jsondump02e.wt'
    table_uri6 = 'table:jsondump02f.wt'

    def set_kv(self, uri, key, val):
        cursor = self.session.open_cursor(uri, None, None)
        try:
            cursor[key] = val
        finally:
            cursor.close()

    def set_kv2(self, uri, key, val1, val2):
        cursor = self.session.open_cursor(uri, None, None)
        try:
            cursor[key] = (val1, val2)
        finally:
            cursor.close()

    # Populate table_uri4 with squares and cubes of row numbers
    def populate_squarecube(self, uri):
        cursor = self.session.open_cursor(uri, None, None)
        for i in range(1, 5):
            square = i * i
            cube = square * i
            cursor[(i, 'key' + str(i))] = \
                ('val' + str(square), square, 'val' + str(cube), cube)
        cursor.close()

    # Check the result of using a JSON cursor on the URI.
    def check_json(self, uri, expect):
        cursor = self.session.open_cursor(uri, None, 'dump=json')
        pos = 0
        for k,v in cursor:
            self.assertEqual(k, expect[pos][0])
            self.assertEqual(v, expect[pos][1])
            pos += 1
        self.assertEqual(pos, len(expect))
        cursor.close()

    # Check the result of using a JSON cursor on the URI.
    def load_json(self, uri, inserts):
        cursor = self.session.open_cursor(uri, None, 'dump=json')
        pos = 0
        try:
            for insert in inserts:
                cursor[insert[0]] = insert[1]
        finally:
            cursor.close()

    def test_json_cursor(self):
        """
        Create JSON cursors and test them directly, also test
        dump/load commands.
        """
        extra_params = ',allocation_size=512,' +\
            'internal_page_max=16384,leaf_page_max=131072'
        self.session.create(self.table_uri1,
            'key_format=S,value_format=S' + extra_params)
        self.session.create(self.table_uri2,
            'key_format=S,value_format=8tS' + extra_params)
        self.session.create(self.table_uri3,
            'key_format=r,value_format=u' + extra_params)
        self.session.create(self.table_uri4,
                            "key_format=iS,value_format=SiSi,"
                            "columns=(ikey,Skey,S1,i2,S3,i4),colgroups=(c1,c2)")
        cg = 'colgroup:' + self.basename_uri4
        self.session.create(cg + ":c1", "columns=(S1,i2)")
        self.session.create(cg + ":c2", "columns=(S3,i4)")
        uri4index1 = 'index:' + self.basename_uri4 + ':by-Skey'
        uri4index2 = 'index:' + self.basename_uri4 + ':by-S3'
        uri4index3 = 'index:' + self.basename_uri4 + ':by-i2i4'
        self.session.create(uri4index1, "columns=(Skey)")
        self.session.create(uri4index2, "columns=(S3)")
        self.session.create(uri4index3, "columns=(i2,i4)")

        self.set_kv(self.table_uri1, 'KEY000', 'string value')
        self.set_kv(self.table_uri1, 'KEY001', '\'\"({[]})\"\'\\, etc. allowed')
        # \u03c0 is pi in Unicode, converted by Python to UTF-8: 0xcf 0x80.
        # Here's how UTF-8 might be used.
        self.set_kv(self.table_uri1, 'KEY002', u'\u03c0'.encode('utf-8'))
        # 0xf5-0xff are illegal in Unicode, but may occur legally in C strings.
        self.set_kv(self.table_uri1, 'KEY003', '\xff\xfe')
        self.set_kv2(self.table_uri2, 'KEY000', 123, 'str0')
        self.set_kv2(self.table_uri2, 'KEY001', 234, 'str1')
        self.set_kv(self.table_uri3, 1, '\x01\x02\x03')
        self.set_kv(self.table_uri3, 2, '\x77\x88\x99\x00\xff\xfe')
        self.populate_squarecube(self.table_uri4)

        table1_json =  (
            ('"key0" : "KEY000"', '"value0" : "string value"'),
            ('"key0" : "KEY001"', '"value0" : ' +
             '"\'\\\"({[]})\\\"\'\\\\, etc. allowed"'),
            ('"key0" : "KEY002"', '"value0" : "\\u00cf\\u0080"'),
            ('"key0" : "KEY003"', '"value0" : "\\u00ff\\u00fe"'))
        self.check_json(self.table_uri1, table1_json)

        self.session.truncate(self.table_uri1, None, None, None)
        self.load_json(self.table_uri1, table1_json)
        self.check_json(self.table_uri1, table1_json)

        table2_json =  (
            ('"key0" : "KEY000"', '"value0" : 123,\n"value1" : "str0"'),
            ('"key0" : "KEY001"', '"value0" : 234,\n"value1" : "str1"'))
        self.check_json(self.table_uri2, table2_json)
        self.session.truncate(self.table_uri2, None, None, None)
        self.load_json(self.table_uri2, table2_json)
        self.check_json(self.table_uri2, table2_json)
        self.session.truncate(self.table_uri2, None, None, None)

        # bad tokens
        self.assertRaisesWithMessage(wiredtiger.WiredTigerError,
            lambda: self.load_json(self.table_uri2,
              (('<>abc?', '9'),)),
            '/unknown token/')

        # bad tokens
        self.assertRaisesWithMessage(wiredtiger.WiredTigerError,
            lambda: self.load_json(self.table_uri2,
              (('"abc\u"', ''),)),
            '/invalid Unicode/')

        # bad tokens
        self.assertRaisesWithMessage(wiredtiger.WiredTigerError,
            lambda: self.load_json(self.table_uri2,
              (('"abc', ''),)),
            '/unterminated string/')

        # bad syntax
        self.assertRaisesWithMessage(wiredtiger.WiredTigerError,
            lambda: self.load_json(self.table_uri2,
              (('"stuff" "jibberish"', '"value0" "more jibberish"'),)),
            '/expected key name.*\"key0\"/')

        # bad types
        self.assertRaisesWithMessage(wiredtiger.WiredTigerError,
            lambda: self.load_json(self.table_uri2,
              (('"key0" : "KEY002"', '"value0" : "xyz",\n"value1" : "str0"'),)),
            '/expected unsigned JSON <int>, got <string>/')

        # bad types
        self.assertRaisesWithMessage(wiredtiger.WiredTigerError,
            lambda: self.load_json(self.table_uri2,
              (('"key0" : "KEY002"', '"value0" : 123,\n"value1" : 456'),)),
            '/expected JSON <string>, got <integer>/')

        # extra stuff
        self.assertRaisesWithMessage(wiredtiger.WiredTigerError,
            lambda: self.load_json(self.table_uri2,
              (('"key0" : "KEY002"',
                '"value0" : 123,\n"value1" : "str0",'),)),
            '/expected JSON <EOF>, got \',\'/')

        # fields out of order currently not supported
        self.assertRaisesWithMessage(wiredtiger.WiredTigerError,
            lambda: self.load_json(self.table_uri2,
              (('"key0" : "KEY002"', '"value1" : "str0",\n"value0" : 123'),)),
            '/expected value name.*\"value0\"/')

        # various invalid unicode
        invalid_unicode = (
            '\\u', '\\ux', '\\u0', '\\u0F', '\\u0FA', '\\u0FAx',  '\\u0FA\\x')
        for uni in invalid_unicode:
            self.assertRaisesWithMessage(wiredtiger.WiredTigerError,
                lambda: self.load_json(self.table_uri2,
                  (('"key0" : "KEY002"', '"value0" : 123,\n"value1" : "'
                    + uni + '"'),)),
                '/invalid Unicode/')

        # this one should work
        self.load_json(self.table_uri2,
              (('"key0" : "KEY002"', '"value0" : 34,\n"value1" : "str2"'),))

        # extraneous/missing space is okay
        self.load_json(self.table_uri2,
              (('  "key0"\n:\t"KEY003"    ',
                '"value0":45,"value1"\n\n\r\n:\t\n"str3"'),))

        table2_json =  (
            ('"key0" : "KEY002"', '"value0" : 34,\n"value1" : "str2"'),
            ('"key0" : "KEY003"', '"value0" : 45,\n"value1" : "str3"'))

        table3_json =  (
            ('"key0" : 1', '"value0" : "\\u0001\\u0002\\u0003"'),
            ('"key0" : 2',
             '"value0" : "\\u0077\\u0088\\u0099\\u0000\\u00ff\\u00fe"'))
        self.check_json(self.table_uri3, table3_json)
        table4_json = (
                ('"ikey" : 1,\n"Skey" : "key1"',
                 '"S1" : "val1",\n"i2" : 1,\n"S3" : "val1",\n"i4" : 1'),
                ('"ikey" : 2,\n"Skey" : "key2"',
                 '"S1" : "val4",\n"i2" : 4,\n"S3" : "val8",\n"i4" : 8'),
                ('"ikey" : 3,\n"Skey" : "key3"',
                 '"S1" : "val9",\n"i2" : 9,\n"S3" : "val27",\n"i4" : 27'),
                ('"ikey" : 4,\n"Skey" : "key4"',
                 '"S1" : "val16",\n"i2" : 16,\n"S3" : "val64",\n"i4" : 64'))
        self.check_json(self.table_uri4, table4_json)
        # The dump config currently is not supported for the index type.
        self.check_json(uri4index1, (
                ('"Skey" : "key1"',
                 '"S1" : "val1",\n"i2" : 1,\n"S3" : "val1",\n"i4" : 1'),
                ('"Skey" : "key2"',
                 '"S1" : "val4",\n"i2" : 4,\n"S3" : "val8",\n"i4" : 8'),
                ('"Skey" : "key3"',
                 '"S1" : "val9",\n"i2" : 9,\n"S3" : "val27",\n"i4" : 27'),
                ('"Skey" : "key4"',
                 '"S1" : "val16",\n"i2" : 16,\n"S3" : "val64",\n"i4" : 64')))
        self.check_json(uri4index2, (
                ('"S3" : "val1"',
                 '"S1" : "val1",\n"i2" : 1,\n"S3" : "val1",\n"i4" : 1'),
                ('"S3" : "val27"',
                 '"S1" : "val9",\n"i2" : 9,\n"S3" : "val27",\n"i4" : 27'),
                ('"S3" : "val64"',
                 '"S1" : "val16",\n"i2" : 16,\n"S3" : "val64",\n"i4" : 64'),
                ('"S3" : "val8"',
                 '"S1" : "val4",\n"i2" : 4,\n"S3" : "val8",\n"i4" : 8')))
        self.check_json(uri4index3, (
                ('"i2" : 1,\n"i4" : 1',
                 '"S1" : "val1",\n"i2" : 1,\n"S3" : "val1",\n"i4" : 1'),
                ('"i2" : 4,\n"i4" : 8',
                 '"S1" : "val4",\n"i2" : 4,\n"S3" : "val8",\n"i4" : 8'),
                ('"i2" : 9,\n"i4" : 27',
                 '"S1" : "val9",\n"i2" : 9,\n"S3" : "val27",\n"i4" : 27'),
                ('"i2" : 16,\n"i4" : 64',
                 '"S1" : "val16",\n"i2" : 16,\n"S3" : "val64",\n"i4" : 64')))

        # Dump all the tables into a single file, and also each
        # table into its own file.
        self.runWt(['dump', '-j',
                    self.table_uri1,
                    self.table_uri2,
                    self.table_uri3,
                    self.table_uri4],
                   outfilename='jsondump-all.out')
        self.runWt(['dump', '-j', self.table_uri1], outfilename='jsondump1.out')
        self.runWt(['dump', '-j', self.table_uri2], outfilename='jsondump2.out')
        self.runWt(['dump', '-j', self.table_uri3], outfilename='jsondump3.out')
        self.runWt(['dump', '-j', self.table_uri4], outfilename='jsondump4.out')
        self.session.drop(self.table_uri1)
        self.session.drop(self.table_uri2)
        self.session.drop(self.table_uri3)
        self.session.drop(self.table_uri4)
        self.runWt(['load', '-jf', 'jsondump1.out'])
        self.session.drop(self.table_uri1)
        self.runWt(['load', '-jf', 'jsondump2.out'])
        self.session.drop(self.table_uri2)
        self.runWt(['load', '-jf', 'jsondump3.out'])
        self.session.drop(self.table_uri3)
        self.runWt(['load', '-jf', 'jsondump4.out'])
        self.session.drop(self.table_uri4)

        self.runWt(['load', '-jf', 'jsondump-all.out'])
        self.check_json(self.table_uri1, table1_json)
        self.check_json(self.table_uri2, table2_json)
        self.check_json(self.table_uri3, table3_json)
        self.check_json(self.table_uri4, table4_json)

    # Generate two byte keys that cover some range of byte values.
    # For simplicity, the keys are monotonically increasing.
    # A null byte is disallowed in a string key, so we don't use it.
    def generate_key(self, i, k):
        k[0] = ((i & 0xffc0) >> 6) + 1
        k[1] = (i & 0x3f) + 1

    # Generate three byte values:
    #    i==0  :  v:[0x00, 0x01, 0x02]
    #    i==1  :  v:[0x01, 0x02, 0x03]
    # etc.
    # A null byte is disallowed in a string value, it is replaced by 'X'
    def generate_value(self, i, v, isstring):
        for j in range(0, 3):
            val = (i + j) % 256
            if isstring and val == 0:
                val = 88  # 'X'
            v[j] = val

    def test_json_all_bytes(self):
        """
        Test the generated JSON for all byte values in byte array and
        string formats.
        """
        self.session.create(self.table_uri5, 'key_format=u,value_format=u')
        self.session.create(self.table_uri6, 'key_format=S,value_format=S')

        c5 = self.session.open_cursor(self.table_uri5, None, None)
        c6 = self.session.open_cursor(self.table_uri6, None, None)
        k = bytearray(b'\x00\x00')
        v = bytearray(b'\x00\x00\x00')
        for i in range(0, 512):
            self.generate_key(i, k)
            self.generate_value(i, v, False)
            c5[str(k)] = str(v)
            self.generate_value(i, v, True)   # no embedded nuls
            c6[str(k)] = str(v)
        c5.close()
        c6.close()

        # Build table5_json, we want it to look like this:
        #    ('"key0" : "\u0001\u0001"', '"value0" : "\u0000\u0001\u0002"'),
        #    ('"key0" : "\u0001\u0002"', '"value0" : "\u0001\u0002\u0003"'))
        #    ('"key0" : "\u0001\u0003"', '"value0" : "\u0003\u0003\u0004"'))
        #    ...
        # table6_json is similar, except that printable values like '\u0041'
        # would appear as 'A'.  The string type cannot have embedded nulls,
        # so '\u0000' in table6_json appears instead as an 'X'.
        #
        # Start by creating two tables of individual Unicode values.
        # bin_unicode[] contains only the \u escape sequences.
        # mix_unicode[] contains printable characters or \t \n etc. escapes
        bin_unicode = []
        mix_unicode = []
        for i in range(0, 256):
            u = "\\u00" + hex(256 + i)[3:]  # e.g. "\u00ab")
            bin_unicode.append(u)
            mix_unicode.append(u)
        for i in range(0x20, 0x7f):
            mix_unicode[i] = chr(i)
        mix_unicode[ord('"')] = '\\"'
        mix_unicode[ord('\\')] = '\\\\'
        mix_unicode[ord('\f')] = '\\f'
        mix_unicode[ord('\n')] = '\\n'
        mix_unicode[ord('\r')] = '\\r'
        mix_unicode[ord('\t')] = '\\t'

        table5_json = []
        table6_json = []
        for i in range(0, 512):
            self.generate_key(i, k)
            self.generate_value(i, v, False)
            j = i if (i > 0 and i < 254) or (i > 256 and i < 510) else 88
            table5_json.append(('"key0" : "' + bin_unicode[k[0]] +
                                bin_unicode[k[1]] + '"',
                                '"value0" : "' + bin_unicode[v[0]] +
                                bin_unicode[v[1]] +
                                bin_unicode[v[2]] + '"'))
            self.generate_value(i, v, True)
            table6_json.append(('"key0" : "' + mix_unicode[k[0]] +
                                mix_unicode[k[1]] + '"',
                                '"value0" : "' + mix_unicode[v[0]] +
                                mix_unicode[v[1]] +
                                mix_unicode[v[2]] + '"'))

        self.check_json(self.table_uri5, table5_json)
        self.check_json(self.table_uri6, table6_json)

        self.session.truncate(self.table_uri5, None, None, None)
        self.session.truncate(self.table_uri6, None, None, None)
        self.load_json(self.table_uri5, table5_json)
        self.load_json(self.table_uri6, table6_json)
        self.check_json(self.table_uri5, table5_json)
        self.check_json(self.table_uri6, table6_json)

        self.runWt(['dump', '-j', self.table_uri5], outfilename='jsondump5.out')
        self.runWt(['dump', '-j', self.table_uri6], outfilename='jsondump6.out')
        self.session.drop(self.table_uri5)
        self.session.drop(self.table_uri6)
        self.runWt(['load', '-jf', 'jsondump5.out'])
        self.runWt(['load', '-jf', 'jsondump6.out'])
        self.session.drop(self.table_uri5)
        self.session.drop(self.table_uri6)

if __name__ == '__main__':
    wttest.run()



"""
Exceptions raised by resmoke.py.
"""


class ResmokeError(Exception):
    """
    Base class for all resmoke.py exceptions.
    """
    pass


class StopExecution(ResmokeError):
    """
    Exception that is raised when resmoke.py should stop executing tests
    if failing fast is enabled.
    """
    pass


class UserInterrupt(StopExecution):
    """
    Exception that is raised when a user signals resmoke.py to
    unconditionally stop executing tests.
    """
    pass


class TestFailure(ResmokeError):
    """
    Exception that is raised by a hook in the after_test method if it
    determines the the previous test should be marked as a failure.
    """
    pass


class ServerFailure(TestFailure):
    """
    Exception that is raised by a hook in the after_test method if it
    detects that the fixture did not exit cleanly and should be marked
    as a failure.
    """
    pass


class PortAllocationError(ResmokeError):
    """
    Exception that is raised by the PortAllocator if a port is requested
    outside of the range of valid ports, or if a fixture requests more
    ports than were reserved for that job.
    """
    pass



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import wiredtiger, wttest
from wtscenario import make_scenarios

# test_join01.py
#    Join operations
# Basic tests for join
class test_join01(wttest.WiredTigerTestCase):
    nentries = 100

    type_scen = [
        ('table', dict(ref='table')),
        ('index', dict(ref='index'))
    ]
    bloom0_scen = [
        ('bloom0=0', dict(joincfg0='')),
        ('bloom0=1000', dict(joincfg0=',strategy=bloom,count=1000')),
        ('bloom0=10000', dict(joincfg0=',strategy=bloom,count=10000')),
    ]
    bloom1_scen = [
        ('bloom1=0', dict(joincfg1='')),
        ('bloom1=1000', dict(joincfg1=',strategy=bloom,count=1000')),
        ('bloom1=10000', dict(joincfg1=',strategy=bloom,count=10000')),
    ]
    projection_scen = [
        ('no-projection', dict(do_proj=False)),
        ('projection', dict(do_proj=True))
    ]
    nested_scen = [
        ('simple', dict(do_nested=False)),
        ('nested', dict(do_nested=True))
    ]
    stats_scen = [
        ('no-stats', dict(do_stats=False)),
        ('stats', dict(do_stats=True))
    ]
    order_scen = [
        ('order=0', dict(join_order=0)),
        ('order=1', dict(join_order=1)),
        ('order=2', dict(join_order=2)),
        ('order=3', dict(join_order=3)),
    ]
    scenarios = make_scenarios(type_scen, bloom0_scen, bloom1_scen,
                               projection_scen, nested_scen, stats_scen,
                               order_scen)

    # We need statistics for these tests.
    conn_config = 'statistics=(all)'

    def gen_key(self, i):
        return [ i + 1 ]

    def gen_values(self, i):
        s = str(i)
        rs = s[::-1]
        sort3 = (self.nentries * (i % 3)) + i    # multiples of 3 sort first
        return [s, rs, sort3]

    # Common function for testing iteration of join cursors
    def iter_common(self, jc, do_proj, do_nested, join_order):
        # See comments in join_common()
        # The order that the results are seen depends on
        # the ordering of the joins.  Specifically, the first
        # join drives the order that results are seen.
        if do_nested:
            if join_order == 0:
                expect = [73, 82, 83, 92]
            elif join_order == 1:
                expect = [73, 82, 83, 92]
            elif join_order == 2:
                expect = [82, 92, 73, 83]
            elif join_order == 3:
                expect = [92, 73, 82, 83]
        else:
            if join_order == 0:
                expect = [73, 82, 62, 83, 92]
            elif join_order == 1:
                expect = [62, 73, 82, 83, 92]
            elif join_order == 2:
                expect = [62, 82, 92, 73, 83]
            elif join_order == 3:
                expect = [73, 82, 62, 83, 92]
        while jc.next() == 0:
            [k] = jc.get_keys()
            i = k - 1
            if do_proj:  # our projection test simply reverses the values
                [v2,v1,v0] = jc.get_values()
            else:
                [v0,v1,v2] = jc.get_values()
            self.assertEquals(self.gen_values(i), [v0,v1,v2])
            if len(expect) == 0 or i != expect[0]:
                self.tty('ERROR: ' + str(i) + ' is not next in: ' +
                         str(expect))
                self.tty('JOIN ORDER=' + str(join_order) + ', NESTED=' + str(do_nested))
            self.assertTrue(i == expect[0])
            expect.remove(i)
        self.assertEquals(0, len(expect))

    # Stats are collected twice: after iterating
    # through the join cursor once, and secondly after resetting
    # the join cursor and iterating again.
    def stats(self, jc, which):
        statcur = self.session.open_cursor('statistics:join', jc, None)
        # pick a stat we always expect to see
        statdesc = 'bloom filter false positives'
        expectstats = [
            'join: index:join01:index1: ' + statdesc,
            'join: index:join01:index2: ' + statdesc ]
        if self.ref == 'index':
            expectstats.append('join: index:join01:index0: ' + statdesc)
        elif self.do_proj:
            expectstats.append('join: table:join01(v2,v1,v0): ' + statdesc)
        else:
            expectstats.append('join: table:join01: ' + statdesc)
        self.check_stats(statcur, expectstats)
        statcur.reset()
        self.check_stats(statcur, expectstats)
        statcur.close()

    def statstr_to_int(self, str):
        """
        Convert a statistics value string, which may be in either form:
        '12345' or '33M (33604836)'
        """
        parts = str.rpartition('(')
        return int(parts[2].rstrip(')'))

    # All of the expect strings should appear
    def check_stats(self, statcursor, expectstats):
        stringclass = ''.__class__
        intclass = (0).__class__

        # Reset the cursor, we're called multiple times.
        statcursor.reset()

        self.printVerbose(3, 'statistics:')
        for id, desc, valstr, val in statcursor:
            self.assertEqual(type(desc), stringclass)
            self.assertEqual(type(valstr), stringclass)
            self.assertEqual(type(val), intclass)
            self.assertEqual(val, self.statstr_to_int(valstr))
            self.printVerbose(3, '  stat: \'' + desc + '\', \'' +
                              valstr + '\', ' + str(val))
            if desc in expectstats:
                expectstats.remove(desc)

        self.assertTrue(len(expectstats) == 0,
                        'missing expected values in stats: ' + str(expectstats))

    def session_record_join(self, jc, refc, config, order, joins):
        joins.append([order, [jc, refc, config]])

    def session_play_one_join(self, firsturi, jc, refc, config):
        if refc.uri == firsturi and config != None:
            config = config.replace('strategy=bloom','')
        #self.tty('->join(jc, uri="' + refc.uri +
        #         '", config="' + str(config) + '"')
        self.session.join(jc, refc, config)

    def session_play_joins(self, joins, join_order):
        #self.tty('->')
        firsturi = None
        for [i, joinargs] in joins:
            if i >= join_order:
                if firsturi == None:
                    firsturi = joinargs[1].uri
                self.session_play_one_join(firsturi, *joinargs)
        for [i, joinargs] in joins:
            if i < join_order:
                if firsturi == None:
                    firsturi = joinargs[1].uri
                self.session_play_one_join(firsturi, *joinargs)

    # Common function for testing the most basic functionality
    # of joins
    def test_join(self):
        joincfg0 = self.joincfg0
        joincfg1 = self.joincfg1
        do_proj = self.do_proj
        do_nested = self.do_nested
        do_stats = self.do_stats
        join_order = self.join_order
        #self.tty('join_common(' + joincfg0 + ',' + joincfg1 + ',' +
        #         str(do_proj) + ',' + str(do_nested) + ',' +
        #         str(do_stats) + ',' + str(join_order) + ')')

        closeme = []
        joins = []   # cursors to be joined

        self.session.create('table:join01', 'key_format=r' +
                            ',value_format=SSi,columns=(k,v0,v1,v2)')
        self.session.create('index:join01:index0','columns=(v0)')
        self.session.create('index:join01:index1','columns=(v1)')
        self.session.create('index:join01:index2','columns=(v2)')

        c = self.session.open_cursor('table:join01', None, None)
        for i in range(0, self.nentries):
            c.set_key(*self.gen_key(i))
            c.set_value(*self.gen_values(i))
            c.insert()
        c.close()

        if do_proj:
            proj_suffix = '(v2,v1,v0)'  # Reversed values
        else:
            proj_suffix = ''            # Default projection (v0,v1,v2)

        # We join on index2 first, not using bloom indices.
        # This defines the order that items are returned.
        # index2 sorts multiples of 3 first (see gen_values())
        # and by using 'gt' and key 99, we'll skip multiples of 3,
        # and examine primary keys 2,5,8,...,95,98,1,4,7,...,94,97.
        jc = self.session.open_cursor('join:table:join01' + proj_suffix,
                                      None, None)
        # Adding a projection to a reference cursor should be allowed.
        c2 = self.session.open_cursor('index:join01:index2(v1)', None, None)
        c2.set_key(99)   # skips all entries w/ primary key divisible by three
        self.assertEquals(0, c2.search())
        self.session_record_join(jc, c2, 'compare=gt', 0, joins)

        # Then select all the numbers 0-99 whose string representation
        # sort >= '60'.
        if self.ref == 'index':
            c0 = self.session.open_cursor('index:join01:index0', None, None)
            c0.set_key('60')
        else:
            c0 = self.session.open_cursor('table:join01', None, None)
            c0.set_key(60)
        self.assertEquals(0, c0.search())
        self.session_record_join(jc, c0, 'compare=ge' + joincfg0, 1, joins)

        # Then select all numbers whose reverse string representation
        # is in '20' < x < '40'.
        c1a = self.session.open_cursor('index:join01:index1(v1)', None, None)
        c1a.set_key('21')
        self.assertEquals(0, c1a.search())
        self.session_record_join(jc, c1a, 'compare=gt' + joincfg1, 2, joins)

        c1b = self.session.open_cursor('index:join01:index1(v1)', None, None)
        c1b.set_key('41')
        self.assertEquals(0, c1b.search())
        self.session_record_join(jc, c1b, 'compare=lt' + joincfg1, 2, joins)

        # Numbers that satisfy these 3 conditions (with ordering implied by c2):
        #    [73, 82, 62, 83, 92].
        #
        # After iterating, we should be able to reset and iterate again.
        if do_nested:
            # To test nesting, we create two new levels of conditions:
            #
            #     x == 72 or x == 73 or x == 82 or x == 83 or
            #       (x >= 90 and x <= 99)
            #
            # that will get AND-ed into our existing join.  The expected
            # result is   [73, 82, 83, 92].
            #
            # We don't specify the projection here, it should be picked up
            # from the 'enclosing' join.
            nest1 = self.session.open_cursor('join:table:join01', None, None)
            nest2 = self.session.open_cursor('join:table:join01', None, None)

            nc = self.session.open_cursor('index:join01:index0', None, None)
            nc.set_key('90')
            self.assertEquals(0, nc.search())
            self.session.join(nest2, nc, 'compare=ge')  # joincfg left out
            closeme.append(nc)

            nc = self.session.open_cursor('index:join01:index0', None, None)
            nc.set_key('99')
            self.assertEquals(0, nc.search())
            self.session.join(nest2, nc, 'compare=le')
            closeme.append(nc)

            self.session.join(nest1, nest2, "operation=or")

            for val in [ '72', '73', '82', '83' ]:
                nc = self.session.open_cursor('index:join01:index0', None, None)
                nc.set_key(val)
                self.assertEquals(0, nc.search())
                self.session.join(nest1, nc, 'compare=eq,operation=or' +
                                  joincfg0)
                closeme.append(nc)
            self.session_record_join(jc, nest1, None, 3, joins)

        self.session_play_joins(joins, join_order)
        self.iter_common(jc, do_proj, do_nested, join_order)
        if do_stats:
            self.stats(jc, 0)
        jc.reset()
        self.iter_common(jc, do_proj, do_nested, join_order)
        if do_stats:
            self.stats(jc, 1)
        jc.reset()
        self.iter_common(jc, do_proj, do_nested, join_order)
        if do_stats:
            self.stats(jc, 2)
        jc.reset()
        self.iter_common(jc, do_proj, do_nested, join_order)

        jc.close()
        c2.close()
        c1a.close()
        c1b.close()
        c0.close()
        if do_nested:
            nest1.close()
            nest2.close()
            for c in closeme:
                c.close()
        self.session.drop('table:join01')

if __name__ == '__main__':
    wttest.run()



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import helper, wiredtiger, wttest
from wiredtiger import stat
from helper import key_populate, simple_populate
from wtscenario import make_scenarios

# test_stat01.py
#    Statistics operations
class test_stat01(wttest.WiredTigerTestCase):
    """
    Test statistics
    """

    config = 'internal_page_max=4K,leaf_page_max=8K'
    nentries = 25

    types = [
        ('file', dict(uri='file:test_stat01.wt')),
        ('table', dict(uri='table:test_stat01.wt'))
    ]
    keyfmt = [
        ('recno', dict(keyfmt='r')),
        ('string', dict(keyfmt='S')),
    ]
    scenarios = make_scenarios(types, keyfmt)

    conn_config = 'statistics=(all)'

    def statstr_to_int(self, str):
        """
        Convert a statistics value string, which may be in either form:
        '12345' or '33M (33604836)'
        """
        parts = str.rpartition('(')
        return int(parts[2].rstrip(')'))

    # Do a quick check of the entries in the the stats cursor, the "lookfor"
    # string should appear with a minimum value of least "min".
    def check_stats(self, statcursor, min, lookfor):
        stringclass = ''.__class__
        intclass = (0).__class__

        # Reset the cursor, we're called multiple times.
        statcursor.reset()

        found = False
        foundval = 0
        for id, desc, valstr, val in statcursor:
            self.assertEqual(type(desc), stringclass)
            self.assertEqual(type(valstr), stringclass)
            self.assertEqual(type(val), intclass)
            self.assertEqual(val, self.statstr_to_int(valstr))
            self.printVerbose(2, '  stat: \'' + desc + '\', \'' +
                              valstr + '\', ' + str(val))
            if desc == lookfor:
                found = True
                foundval = val

        self.assertTrue(found, 'in stats, did not see: ' + lookfor)
        self.assertTrue(foundval >= min)

    # Test simple connection statistics.
    def test_basic_conn_stats(self):
        # Build an object and force some writes.
        config = self.config + ',key_format=' + self.keyfmt
        simple_populate(self, self.uri, config, 1000)
        self.session.checkpoint(None)

        # See that we can get a specific stat value by its key and verify its
        # entry is self-consistent.
        allstat_cursor = self.session.open_cursor('statistics:', None, None)
        self.check_stats(allstat_cursor, 10, 'block-manager: blocks written')

        values = allstat_cursor[stat.conn.block_write]
        self.assertEqual(values[0], 'block-manager: blocks written')
        val = self.statstr_to_int(values[1])
        self.assertEqual(val, values[2])
        allstat_cursor.close()

    # Test simple object statistics.
    def test_basic_data_source_stats(self):
        # Build an object.
        config = self.config + ',key_format=' + self.keyfmt
        self.session.create(self.uri, config)
        cursor = self.session.open_cursor(self.uri, None, None)
        value = ""
        for i in range(1, self.nentries):
            value = value + 1000 * "a"
            cursor[key_populate(cursor, i)] = value
        cursor.close()

        # Force the object to disk, otherwise we can't check the overflow count.
        self.reopen_conn()

        # See that we can get a specific stat value by its key and verify its
        # entry is self-consistent.
        cursor = self.session.open_cursor('statistics:' + self.uri, None, None)
        self.check_stats(cursor, 8192, 'btree: maximum leaf page size')
        self.check_stats(cursor, 4096, 'btree: maximum internal page size')
        self.check_stats(cursor, 10, 'btree: overflow pages')

        values = cursor[stat.dsrc.btree_overflow]
        self.assertEqual(values[0], 'btree: overflow pages')
        val = self.statstr_to_int(values[1])
        self.assertEqual(val, values[2])
        cursor.close()

        cursor = self.session.open_cursor(
            'statistics:' + self.uri, None, "statistics=(size)")
        values = cursor[stat.dsrc.block_size]
        self.assertNotEqual(values[2], 0)
        cursor.close()

    # Test simple per-checkpoint statistics.
    def test_checkpoint_stats(self):
        for name in ('first', 'second', 'third'):
            config = self.config + ',key_format=' + self.keyfmt
            helper.simple_populate(self, self.uri, config, self.nentries)
            self.session.checkpoint('name=' + name)
            cursor = self.session.open_cursor(
                'statistics:' + self.uri, None, 'checkpoint=' + name)
            self.assertEqual(
                cursor[stat.dsrc.btree_entries][2], self.nentries)
            cursor.close()

    def test_missing_file_stats(self):
        self.assertRaises(wiredtiger.WiredTigerError, lambda:
            self.session.open_cursor('statistics:file:DoesNotExist'))

if __name__ == '__main__':
    wttest.run()



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
# test_txn08.py
# Printlog: test Unicode output
#

import fnmatch, os, shutil, run, time
from suite_subprocess import suite_subprocess
from wiredtiger import stat
import wttest

class test_txn08(wttest.WiredTigerTestCase, suite_subprocess):
    logmax = "100K"
    tablename = 'test_txn08'
    uri = 'table:' + tablename

    # Turn on logging for this test.
    def conn_config(self, dir):
        return 'log=(archive=false,enabled,file_max=%s),' % self.logmax + \
            'transaction_sync="(method=dsync,enabled)"'

    def test_printlog_unicode(self):
        # print "Creating %s with config '%s'" % (self.uri, self.create_params)
        create_params = 'key_format=i,value_format=S'
        self.session.create(self.uri, create_params)
        c = self.session.open_cursor(self.uri, None)

        # We want to test some chars that produce Unicode encoding
        # for printlog output.
        value = u'\u0001\u0002abcd\u0003\u0004'

        self.session.begin_transaction()
        for k in range(5):
            c[k] = value

        self.session.commit_transaction()

        #
        # Run printlog and make sure it exits with zero status.
        #
        self.runWt(['printlog'], outfilename='printlog.out')
        self.check_file_contains('printlog.out',
            '\\u0001\\u0002abcd\\u0003\\u0004')
        self.runWt(['printlog', '-x'], outfilename='printlog-hex.out')
        self.check_file_contains('printlog-hex.out',
            '\\u0001\\u0002abcd\\u0003\\u0004')
        self.check_file_contains('printlog-hex.out',
            '0102616263640304')

if __name__ == '__main__':
    wttest.run()



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
# test_bug001.py
#       Regression tests.

import wiredtiger, wttest

# Regression tests.
class test_bug001(wttest.WiredTigerTestCase):

    def create_implicit(self, uri, initial, middle, trailing):
        self.session.create(uri, 'key_format=r,value_format=8t')
        cursor = self.session.open_cursor(uri, None)

        # Create a set of initial implicit records, followed by a set of real
        # records, followed by a set of trailing implicit records.
        r = initial
        for i in range(0, middle):
            r += 1
            cursor[r] = 0xab
        r += trailing
        cursor[r + 1] = 0xbb
        return (cursor)

    # Test a bug where cursor movement inside implicit records failed.
    def test_implicit_record_cursor_movement(self):
        uri = 'file:xxx'
        cursor = self.create_implicit(uri, 0, 50, 20)

        # Check search inside trailing implicit keys.
        for i in range(0, 5):
            self.assertEqual(cursor[60 + i], 0x00)

        # Check cursor next inside trailing implicit keys.
        cursor.set_key(60)
        self.assertEquals(cursor.search(), 0)
        for i in range(0, 5):
            self.assertEqual(cursor.get_key(), 60 + i)
            self.assertEqual(cursor.get_value(), 0x00)
            self.assertEqual(cursor.next(), 0)

        # Check cursor prev inside trailing implicit keys.
        cursor.set_key(60)
        self.assertEquals(cursor.search(), 0)
        for i in range(0, 5):
            self.assertEqual(cursor.get_key(), 60 - i)
            self.assertEqual(cursor.get_value(), 0x00)
            self.assertEqual(cursor.prev(), 0)

        self.assertEquals(cursor.close(), 0)
        self.session.drop(uri)
        cursor = self.create_implicit(uri, 20, 50, 0)

        # Check search inside leading implicit keys.
        for i in range(0, 5):
            self.assertEqual(cursor[10 + i], 0x00)

        # Check cursor next inside leading implicit keys.
        cursor.set_key(10)
        self.assertEquals(cursor.search(), 0)
        for i in range(0, 5):
            self.assertEqual(cursor.get_key(), 10 + i)
            self.assertEqual(cursor.get_value(), 0x00)
            self.assertEqual(cursor.next(), 0)

        # Check cursor prev inside leading implicit keys.
        cursor.set_key(10)
        self.assertEquals(cursor.search(), 0)
        for i in range(0, 5):
            self.assertEqual(cursor.get_key(), 10 - i)
            self.assertEqual(cursor.get_value(), 0x00)
            self.assertEqual(cursor.prev(), 0)

        self.assertEquals(cursor.close(), 0)
        self.session.drop(uri)

    # Test a bug where cursor remove inside implicit records looped infinitely.
    def test_implicit_record_cursor_remove(self):
        uri='file:xxx'
        cursor = self.create_implicit(uri, 0, 50, 20)

        # Check cursor next/remove inside trailing implicit keys.
        cursor.set_key(62)
        self.assertEquals(cursor.search(), 0)
        for i in range(1, 5):
            self.assertEquals(cursor.next(), 0)
            self.assertEqual(cursor.get_key(), 62 + i)
            self.assertEqual(cursor.get_value(), 0x00)
            self.assertEquals(cursor.remove(), 0)

        # Check cursor prev/remove inside trailing implicit keys.
        cursor.set_key(68)
        self.assertEquals(cursor.search(), 0)
        for i in range(1, 5):
            self.assertEquals(cursor.prev(), 0)
            self.assertEqual(cursor.get_key(), 68 - i)
            self.assertEqual(cursor.get_value(), 0x00)
            self.assertEquals(cursor.remove(), 0)

        self.assertEquals(cursor.close(), 0)
        self.session.drop(uri)
        cursor = self.create_implicit(uri, 20, 50, 0)

        # Check cursor next/remove inside leading implicit keys.
        cursor.set_key(2)
        self.assertEquals(cursor.search(), 0)
        for i in range(1, 5):
            self.assertEquals(cursor.next(), 0)
            self.assertEqual(cursor.get_key(), 2 + i)
            self.assertEqual(cursor.get_value(), 0x00)
            self.assertEquals(cursor.remove(), 0)

        # Check cursor prev/remove inside leading implicit keys.
        cursor.set_key(18)
        self.assertEquals(cursor.search(), 0)
        for i in range(1, 5):
            self.assertEquals(cursor.prev(), 0)
            self.assertEqual(cursor.get_key(), 18 - i)
            self.assertEqual(cursor.get_value(), 0x00)
            self.assertEquals(cursor.remove(), 0)

        self.assertEquals(cursor.close(), 0)
        self.session.drop(uri)

if __name__ == '__main__':
    wttest.run()



"""
Workaround for having too many threads running on 32-bit systems when
logging to buildlogger that still allows periodically flushing messages
to the buildlogger server.

This is because a utils.timer.AlarmClock instance is used for each
buildlogger.BuildloggerTestHandler, but only dismiss()ed when the Python
process is about to exit.
"""

from __future__ import absolute_import

import threading

from ..utils import queue


_LOGGER_QUEUE = queue.Queue()

_FLUSH_THREAD_LOCK = threading.Lock()
_FLUSH_THREAD = None


def start_thread():
    """
    Starts the flush thread.
    """

    global _FLUSH_THREAD
    with _FLUSH_THREAD_LOCK:
        if _FLUSH_THREAD is not None:
            raise ValueError("FlushThread has already been started")

        _FLUSH_THREAD = _FlushThread()
        _FLUSH_THREAD.start()


def stop_thread():
    """
    Signals the flush thread to stop and waits until it does.
    """

    with _FLUSH_THREAD_LOCK:
        if _FLUSH_THREAD is None:
            raise ValueError("FlushThread hasn't been started")

    # Add sentinel value to indicate when there are no more loggers to process.
    _LOGGER_QUEUE.put(None)
    _FLUSH_THREAD.join()


def close_later(logger):
    """
    Adds 'logger' to the queue so that it is closed later by the flush
    thread.
    """
    _LOGGER_QUEUE.put(logger)


class _FlushThread(threading.Thread):
    """
    Asynchronously flushes and closes logging handlers.
    """

    def __init__(self):
        """
        Initializes the flush thread.
        """

        threading.Thread.__init__(self, name="FlushThread")
        # Do not wait to flush the logs if interrupted by the user.
        self.daemon = True

    def run(self):
        """
        Continuously shuts down loggers from the queue.
        """

        while True:
            logger = _LOGGER_QUEUE.get()
            try:
                if logger is None:
                    # Sentinel value received, so exit.
                    break
                _FlushThread._shutdown_logger(logger)
            finally:
                _LOGGER_QUEUE.task_done()

    @staticmethod
    def _shutdown_logger(logger):
        """
        Flushes and closes all handlers of 'logger'.
        """

        for handler in logger.handlers:
            handler.flush()
            handler.close()



# Copyright (C) 2015 MongoDB Inc.
#
# This program is free software: you can redistribute it and/or  modify
# it under the terms of the GNU Affero General Public License, version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import SCons
import subprocess

def _detect(env):
    try:
        abidw = env['ABIDW']
        if not abidw:
            return None
        return abidw
    except KeyError:
        pass

    return env.WhereIs('abidw')

def generate(env):

    class AbilinkNode(SCons.Node.FS.File):
        def __init__(self, name, directory, fs):
            SCons.Node.FS.File.__init__(self, name, directory, fs)

        def get_contents(self):
            if not self.rexists():
                return ''

            fname = self.rfile().abspath
            contents = None

            try:
                # TODO: If there were python bindings for libabigail, we
                # could avoid the shell out (and probably be faster, as we
                # could get exactly the information we want).
                contents = subprocess.check_output([env.subst('$ABIDW'), fname])
            except subprocess.CalledProcessError, e:
                # ABIDW sometimes fails. In that case, log an error
                # and fall back to the normal contents
                print "WARNING: ABIDW failed for target %s, please file a bug report" % fname
                try:
                    contents = open(fname, "rb").read()
                except EnvironmentError, e:
                    if not e.filename:
                        e.filename = fname
                    raise

            return contents

        def get_content_hash(self):
            return SCons.Util.MD5signature(self.get_contents())

    env['ABIDW'] = _detect(env)

    def ShlibNode(env, name, directory = None, create = 1):
        return env.fs._lookup(env.subst(name), directory, AbilinkNode, create)

    env.AddMethod(ShlibNode, 'ShlibNode')

    def shlib_target_factory(arg):
        return env.ShlibNode(arg)

    env['BUILDERS']['SharedLibrary'].target_factory = shlib_target_factory

def exists(env):
    result = _detect(env) != None
    return result



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
# test_encrypt03.py
#   Test some error conditions with encryption.
#

import os, run, random
import wiredtiger, wttest
from wtscenario import make_scenarios

# Test basic encryption
class test_encrypt03(wttest.WiredTigerTestCase):

    types = [
        ('table', dict(uri='table:test_encrypt03')),
    ]
    encrypt = [
        ('none', dict( sys_encrypt='none', sys_encrypt_args='',
            file_encrypt='rotn', file_encrypt_args=',keyid=13')),
        # This case is now permitted: it the system encryption is inherited by
        # the table.
        #('noname', dict( sys_encrypt='rotn', sys_encrypt_args=',keyid=11',
        #    file_encrypt='none', file_encrypt_args=',keyid=13')),
    ]
    scenarios = make_scenarios(types, encrypt)

    # Override WiredTigerTestCase, we have extensions.
    def setUpConnectionOpen(self, dir):
        encarg = 'encryption=(name={0}{1}),'.format(
            self.sys_encrypt, self.sys_encrypt_args)
        extarg = self.extensionArg([('encryptors', self.sys_encrypt),
            ('encryptors', self.file_encrypt)])
        self.pr('encarg = ' + encarg + ' extarg = ' + extarg)
        conn = self.wiredtiger_open(dir,
            'create,error_prefix="{0}: ",{1}{2}'.format(
                self.shortid(), encarg, extarg))
        self.pr(`conn`)
        return conn

    # Return the wiredtiger_open extension argument for a shared library.
    def extensionArg(self, exts):
        extfiles = []
        for ext in exts:
            (dirname, name) = ext
            if name != None and name != 'none':
                testdir = os.path.dirname(__file__)
                extdir = os.path.join(run.wt_builddir, 'ext', dirname)
                extfile = os.path.join(
                    extdir, name, '.libs', 'libwiredtiger_' + name + '.so')
                if not os.path.exists(extfile):
                    self.skipTest('extension "' + extfile + '" not built')
                if not extfile in extfiles:
                    extfiles.append(extfile)
        if len(extfiles) == 0:
            return ''
        else:
            return ',extensions=["' + '","'.join(extfiles) + '"]'

    # Create a table with encryption values that are in error.
    def test_encrypt(self):
        params = 'key_format=S,value_format=S,encryption=(name='
        if self.file_encrypt != None:
            params += self.file_encrypt
        if self.file_encrypt_args != None:
            params += ',keyid=' + self.file_encrypt_args
        params += ')'

        # All error messages so far have this in common.
        msg = '/to be set: Invalid argument/'

        self.assertRaisesWithMessage(wiredtiger.WiredTigerError, lambda:
            self.session.create(self.uri, params), msg)

if __name__ == '__main__':
    wttest.run()



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#

import os
from wiredtiger import wiredtiger_open

# Connect to the database and open a session
os.system('rm -rf WT_HOME')
os.makedirs('WT_HOME')

conn = wiredtiger_open('WT_HOME', 'create')
session = conn.open_session()

# Create a simple table
session.create('table:T', 'key_format=S,value_format=S')

# Open a cursor and insert a record
cursor = session.open_cursor('table:T', None)

cursor.set_key('key1')
cursor.set_value('value1')
cursor.insert()

# Iterate through the records
cursor.reset()
for key, value in cursor:
    print('Got record: %s : %s' % (key, value))

conn.close()



#!/usr/bin/env python
"""
eslint.py
 Will download a prebuilt ESLint binary if necessary (i.e. it isn't installed, isn't in the current
 path, or is the wrong version). It works in much the same way as clang_format.py. In lint mode, it
 will lint the files or directory paths passed. In lint-patch mode, for upload.py, it will see if
 there are any candidate files in the supplied patch. Fix mode will run ESLint with the --fix
 option, and that will update the files with missing semicolons and similar repairable issues.
 There is also a -d mode that assumes you only want to run one copy of ESLint per file / directory
 parameter supplied. This lets ESLint search for candidate files to lint.
"""
import Queue
import itertools
import os
import re
import shutil
import string
import subprocess
import sys
import tarfile
import tempfile
import threading
import time
import urllib
from distutils import spawn
from multiprocessing import cpu_count
from optparse import OptionParser

# Get relative imports to work when the package is not installed on the PYTHONPATH.
if __name__ == "__main__" and __package__ is None:
    sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(os.path.realpath(__file__)))))

from buildscripts.resmokelib.utils import globstar
from buildscripts import moduleconfig


##############################################################################
#
# Constants for ESLint
#
#

# Expected version of ESLint.
ESLINT_VERSION = "2.3.0"

# Name of ESLint as a binary.
ESLINT_PROGNAME = "eslint"

# URL location of our provided ESLint binaries.
ESLINT_HTTP_LINUX_CACHE = "https://s3.amazonaws.com/boxes.10gen.com/build/eslint-" + \
                           ESLINT_VERSION + "-linux.tar.gz"
ESLINT_HTTP_DARWIN_CACHE = "https://s3.amazonaws.com/boxes.10gen.com/build/eslint-" + \
                            ESLINT_VERSION + "-darwin.tar.gz"

# Path in the tarball to the ESLint binary.
ESLINT_SOURCE_TAR_BASE = string.Template(ESLINT_PROGNAME + "-$platform-$arch")

# Path to the modules in the mongodb source tree.
# Has to match the string in SConstruct.
MODULE_DIR = "src/mongo/db/modules"

# Copied from python 2.7 version of subprocess.py
# Exception classes used by this module.
class CalledProcessError(Exception):
    """This exception is raised when a process run by check_call() or
    check_output() returns a non-zero exit status.
    The exit status will be stored in the returncode attribute;
    check_output() will also store the output in the output attribute.
    """
    def __init__(self, returncode, cmd, output=None):
        self.returncode = returncode
        self.cmd = cmd
        self.output = output
    def __str__(self):
        return ("Command '%s' returned non-zero exit status %d with output %s" %
            (self.cmd, self.returncode, self.output))


# Copied from python 2.7 version of subprocess.py
def check_output(*popenargs, **kwargs):
    r"""Run command with arguments and return its output as a byte string.

    If the exit code was non-zero it raises a CalledProcessError.  The
    CalledProcessError object will have the return code in the returncode
    attribute and output in the output attribute.

    The arguments are the same as for the Popen constructor.  Example:

    >>> check_output(["ls", "-l", "/dev/null"])
    'crw-rw-rw- 1 root root 1, 3 Oct 18  2007 /dev/null\n'

    The stdout argument is not allowed as it is used internally.
    To capture standard error in the result, use stderr=STDOUT.

    >>> check_output(["/bin/sh", "-c",
    ...               "ls -l non_existent_file ; exit 0"],
    ...              stderr=STDOUT)
    'ls: non_existent_file: No such file or directory\n'
    """
    if 'stdout' in kwargs:
        raise ValueError('stdout argument not allowed, it will be overridden.')
    process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
    output, unused_err = process.communicate()
    retcode = process.poll()
    if retcode:
        cmd = kwargs.get("args")
        if cmd is None:
            cmd = popenargs[0]
        raise CalledProcessError(retcode, cmd, output)
    return output

def callo(args):
    """Call a program, and capture its output
    """
    return check_output(args)

def extract_eslint(tar_path, target_file):
    tarfp = tarfile.open(tar_path)
    for name in tarfp.getnames():
        if name == target_file:
            tarfp.extract(name)
    tarfp.close()

def get_eslint_from_cache(dest_file, platform, arch):
    """Get ESLint binary from mongodb's cache
    """
    # Get URL
    if platform == "Linux":
        url = ESLINT_HTTP_LINUX_CACHE
    elif platform == "Darwin":
        url = ESLINT_HTTP_DARWIN_CACHE
    else:
        raise ValueError('ESLint is not available as a binary for ' + platform)

    dest_dir = tempfile.gettempdir()
    temp_tar_file = os.path.join(dest_dir, "temp.tar.gz")

    # Download the file
    print("Downloading ESLint %s from %s, saving to %s" % (ESLINT_VERSION,
                                                           url, temp_tar_file))
    urllib.urlretrieve(url, temp_tar_file)

    eslint_distfile = ESLINT_SOURCE_TAR_BASE.substitute(platform=platform, arch=arch)
    extract_eslint(temp_tar_file, eslint_distfile)
    shutil.move(eslint_distfile, dest_file)


class ESLint(object):
    """Class encapsulates finding a suitable copy of ESLint, and linting an individual file
    """
    def __init__(self, path, cache_dir):
        eslint_progname = ESLINT_PROGNAME

        # Initialize ESLint configuration information
        if sys.platform.startswith("linux"):
            self.arch = "x86_64"
            self.tar_path = None
        elif sys.platform == "darwin":
            self.arch = "x86_64"
            self.tar_path = None

        self.path = None

        # Find ESLint now
        if path is not None:
            if os.path.isfile(path):
                self.path = path
            else:
                print("WARNING: Could not find ESLint at %s" % (path))

        # Check the environment variable
        if "MONGO_ESLINT" in os.environ:
            self.path = os.environ["MONGO_ESLINT"]

            if self.path and not self._validate_version(warn=True):
                self.path = None

        # Check the user's PATH environment variable now
        if self.path is None:
            self.path = spawn.find_executable(eslint_progname)

            if self.path and not self._validate_version(warn=True):
                self.path = None

        # Have not found it yet, download it from the web
        if self.path is None:
            if not os.path.isdir(cache_dir):
                os.makedirs(cache_dir)

            self.path = os.path.join(cache_dir, eslint_progname)

            if not os.path.isfile(self.path):
                if sys.platform.startswith("linux"):
                    get_eslint_from_cache(self.path, "Linux", self.arch)
                elif sys.platform == "darwin":
                    get_eslint_from_cache(self.path, "Darwin", self.arch)
                else:
                    print("ERROR: eslint.py does not support downloading ESLint " +
                          "on this platform, please install ESLint " + ESLINT_VERSION)
        # Validate we have the correct version
        if not self._validate_version():
            raise ValueError('correct version of ESLint was not found.')

        self.print_lock = threading.Lock()

    def _validate_version(self, warn=False):
        """Validate ESLint is the expected version
        """
        esl_version = callo([self.path, "--version"]).rstrip()
        # Ignore the leading v in the version string.
        if ESLINT_VERSION == esl_version[1:]:
            return True

        if warn:
            print("WARNING: eslint found in path, but incorrect version found at " +
                  self.path + " with version: " + esl_version)
        return False

    def _lint(self, file_name, print_diff):
        """Check the specified file for linting errors
        """
        # ESLint returns non-zero on a linting error. That's all we care about
        # so only enter the printing logic if we have an error.
        try:
            eslint_output = callo([self.path, "-f", "unix", file_name])
        except CalledProcessError as e:
            if print_diff:
                # Take a lock to ensure error messages do not get mixed when printed to the screen
                with self.print_lock:
                    print("ERROR: ESLint found errors in " + file_name)
                    print(e.output)
            return False
        except:
            print("ERROR: ESLint process threw unexpected error", sys.exc_info()[0])
            return False

        return True

    def lint(self, file_name):
        """Check the specified file has no linting errors
        """
        return self._lint(file_name, print_diff=True)

    def autofix(self, file_name):
        """ Run ESLint in fix mode.
        """
        return not subprocess.call([self.path, "--fix", file_name])

def parallel_process(items, func):
    """Run a set of work items to completion
    """
    try:
        cpus = cpu_count()
    except NotImplementedError:
        cpus = 1

    task_queue = Queue.Queue()

    # Use a list so that worker function will capture this variable
    pp_event = threading.Event()
    pp_result = [True]
    pp_lock = threading.Lock()

    def worker():
        """Worker thread to process work items in parallel
        """
        while not pp_event.is_set():
            try:
                item = task_queue.get_nowait()
            except Queue.Empty:
                # if the queue is empty, exit the worker thread
                pp_event.set()
                return

            try:
                ret = func(item)
            finally:
                # Tell the queue we finished with the item
                task_queue.task_done()

            # Return early if we fail, and signal we are done
            if not ret:
                with pp_lock:
                    pp_result[0] = False

                pp_event.set()
                return

    # Enqueue all the work we want to process
    for item in items:
        task_queue.put(item)

    # Process all the work
    threads = []
    for cpu in range(cpus):
        thread = threading.Thread(target=worker)

        thread.daemon = True
        thread.start()
        threads.append(thread)

    # Wait for the threads to finish
    # Loop with a timeout so that we can process Ctrl-C interrupts
    # Note: On Python 2.6 wait always returns None so we check is_set also,
    #  This works because we only set the event once, and never reset it
    while not pp_event.wait(1) and not pp_event.is_set():
        time.sleep(1)

    for thread in threads:
        thread.join()
    return pp_result[0]

def get_base_dir():
    """Get the base directory for mongo repo.
        This script assumes that it is running in buildscripts/, and uses
        that to find the base directory.
    """
    try:
        return subprocess.check_output(['git', 'rev-parse', '--show-toplevel']).rstrip()
    except:
        # We are not in a valid git directory. Use the script path instead.
        return os.path.dirname(os.path.dirname(os.path.realpath(__file__)))

def get_repos():
    """Get a list of linked repos and directories to run ESLint on.
    """
    base_dir = get_base_dir()

    # Get a list of modules
    # TODO: how do we filter rocks, does it matter?
    mongo_modules = moduleconfig.discover_module_directories(
                        os.path.join(base_dir, MODULE_DIR), None)

    paths = [os.path.join(base_dir, MODULE_DIR, m) for m in mongo_modules]

    paths.append(base_dir)

    return [Repo(p) for p in paths]


class Repo(object):
    """Class encapsulates all knowledge about a git repository, and its metadata
        to run ESLint.
    """
    def __init__(self, path):
        self.path = path

        # Get candidate files
        self.candidate_files = self.get_candidate_files()

        self.root = self._get_root()

    def _callgito(self, args):
        """Call git for this repository
        """
        # These two flags are the equivalent of -C in newer versions of Git
        # but we use these to support versions back to ~1.8
        return callo(['git', '--git-dir', os.path.join(self.path, ".git"),
                        '--work-tree', self.path] + args)

    def _get_local_dir(self, path):
        """Get a directory path relative to the git root directory
        """
        if os.path.isabs(path):
            return os.path.relpath(path, self.root)
        return path

    def get_candidates(self, candidates):
        """Get the set of candidate files to check by doing an intersection
        between the input list, and the list of candidates in the repository

        Returns the full path to the files for ESLint to consume.
        """
        # NOTE: Files may have an absolute root (i.e. leading /)

        if candidates is not None and len(candidates) > 0:
            candidates = [self._get_local_dir(f) for f in candidates]
            valid_files = list(set(candidates).intersection(self.get_candidate_files()))
        else:
            valid_files = list(self.get_candidate_files())

        # Get the full file names here
        valid_files = [os.path.normpath(os.path.join(self.root, f)) for f in valid_files]
        return valid_files

    def _get_root(self):
        """Gets the root directory for this repository from git
        """
        gito = self._callgito(['rev-parse', '--show-toplevel'])

        return gito.rstrip()

    def get_candidate_files(self):
        """Query git to get a list of all files in the repo to consider for analysis
        """
        gito = self._callgito(["ls-files"])

        # This allows us to pick all the interesting files
        # in the mongo and mongo-enterprise repos
        file_list = [line.rstrip()
                     for line in gito.splitlines()
                     if "src/mongo" in line or "jstests" in line]

        files_match = re.compile('\\.js$')

        file_list = [a for a in file_list if files_match.search(a)]

        return file_list


def expand_file_string(glob_pattern):
    """Expand a string that represents a set of files
    """
    return [os.path.abspath(f) for f in globstar.iglob(glob_pattern)]

def get_files_to_check(files):
    """Filter the specified list of files to check down to the actual
        list of files that need to be checked."""
    candidates = []

    # Get a list of candidate_files
    candidates = [expand_file_string(f) for f in files]
    candidates = list(itertools.chain.from_iterable(candidates))

    repos = get_repos()

    valid_files = list(itertools.chain.from_iterable([r.get_candidates(candidates) for r in repos]))

    return valid_files

def get_files_to_check_from_patch(patches):
    """Take a patch file generated by git diff, and scan the patch for a list of files to check.
    """
    candidates = []

    # Get a list of candidate_files
    check = re.compile(r"^diff --git a\/([a-z\/\.\-_0-9]+) b\/[a-z\/\.\-_0-9]+")

    lines = []
    for patch in patches:
        with open(patch, "rb") as infile:
            lines += infile.readlines()

    candidates = [check.match(line).group(1) for line in lines if check.match(line)]

    repos = get_repos()

    valid_files = list(itertools.chain.from_iterable([r.get_candidates(candidates) for r in repos]))

    return valid_files

def _get_build_dir():
    """Get the location of the scons build directory in case we need to download ESLint
    """
    return os.path.join(get_base_dir(), "build")

def _lint_files(eslint, files):
    """Lint a list of files with ESLint
    """
    eslint = ESLint(eslint, _get_build_dir())

    lint_clean = parallel_process([os.path.abspath(f) for f in files], eslint.lint)

    if not lint_clean:
        print("ERROR: ESLint found errors. Run ESLint manually to see errors in "\
              "files that were skipped")
        sys.exit(1)

    return True

def lint_patch(eslint, infile):
    """Lint patch command entry point
    """
    files = get_files_to_check_from_patch(infile)

    # Patch may have files that we do not want to check which is fine
    if files:
        return _lint_files(eslint, files)
    return True

def lint(eslint, dirmode, glob):
    """Lint files command entry point
    """
    if dirmode and glob:
        files = glob
    else:
        files = get_files_to_check(glob)

    _lint_files(eslint, files)

    return True

def _autofix_files(eslint, files):
    """Auto-fix the specified files with ESLint.
    """
    eslint = ESLint(eslint, _get_build_dir())

    autofix_clean = parallel_process([os.path.abspath(f) for f in files], eslint.autofix)

    if not autofix_clean:
        print("ERROR: failed to auto-fix files")
        return False

def autofix_func(eslint, dirmode, glob):
    """Auto-fix files command entry point
    """
    if dirmode:
        files = glob
    else:
        files = get_files_to_check(glob)

    return _autofix_files(eslint, files)


def main():
    """Main entry point
    """
    success = False
    usage = "%prog [-e <eslint>] [-d] lint|lint-patch|fix [glob patterns] "
    description = "lint runs ESLint on provided patterns or all .js files under jstests/ "\
                  "and src/mongo. lint-patch runs ESLint against .js files modified in the "\
                  "provided patch file (for upload.py). "\
                  "fix runs ESLint with --fix on provided patterns "\
                  "or files under jstests/ and src/mongo."
    epilog ="*Unless you specify -d a separate ESLint process will be launched for every file"
    parser = OptionParser()
    parser = OptionParser(usage=usage, description=description, epilog=epilog)
    parser.add_option("-e", "--eslint", type="string", dest="eslint",
                      help="Fully qualified path to eslint executable",)
    parser.add_option("-d", "--dirmode", action="store_true", default=True, dest="dirmode",
                      help="Considers the glob patterns as directories and runs ESLint process " \
                           "against each pattern",)

    (options, args) = parser.parse_args(args=sys.argv)

    if len(args) > 1:
        command = args[1]
        searchlist = args[2:]
        if not searchlist:
            searchlist = ["jstests/", "src/mongo/"]

        if command == "lint":
            success = lint(options.eslint, options.dirmode, searchlist)
        elif command == "lint-patch":
            if not args[2:]:
                success = False
                print("You must provide the patch's fully qualified file name with lint-patch")
            else:
                success = lint_patch(options.eslint, searchlist)
        elif command == "fix":
            success = autofix_func(options.eslint, options.dirmode, searchlist)
        else:
            parser.print_help()
    else:
        parser.print_help()

    sys.exit(0 if success else 1)
if __name__ == "__main__":
    main()



#!/usr/bin/python

import os
import subprocess
import sys

SCONS_VERSION = os.environ.get('SCONS_VERSION', "2.5.0")

mongodb_root = os.path.dirname(os.path.dirname(__file__))
scons_dir = os.path.join(mongodb_root, 'src', 'third_party', 'scons-' + SCONS_VERSION)

if sys.platform == 'win32':
    args = [sys.executable, os.path.join(scons_dir, 'scons.py')] + sys.argv[1:]
    sys.exit(subprocess.call(args))
else:
    os.execv(os.path.join(scons_dir, 'scons.py'), sys.argv)



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
# test_durability01.py
#   Durability: make sure the metadata is stable after exclusive operations
#   cause files to be closed.
#

import fnmatch, os, shutil, time
from helper import copy_wiredtiger_home
from suite_subprocess import suite_subprocess
import wttest

class test_durability01(wttest.WiredTigerTestCase, suite_subprocess):
    uri = 'table:test_durability01'
    create_params = 'key_format=i,value_format=i'

    def check_crash_restart(self, olddir, newdir):
        ''' Simulate a crash from olddir and restart in newdir. '''
        # with the connection still open, copy files to new directory
        copy_wiredtiger_home(olddir, newdir)

        # Open the new directory
        conn = self.setUpConnectionOpen(newdir)
        session = self.setUpSessionOpen(conn)
        session.verify(self.uri)
        conn.close()

    def test_durability(self):
        '''Check for missing metadata checkpoints'''

        # Here's the strategy:
        #    - update the table
        #    - verify, which causes the table to be flushed
        #    - copy the database directory (live, simulating a crash)
        #    - verify in the copy
        #    - repeat
        #
        # If the metadata isn't flushed, eventually the metadata we copy will
        # be sufficiently out-of-sync with the data file that it won't verify.
        self.session.create(self.uri, self.create_params)
        for i in range(100):
            c = self.session.open_cursor(self.uri)
            c[i] = i
            c.close()
            if i % 5 == 0:
                self.session.checkpoint()
            else:
                self.session.verify(self.uri)
            self.check_crash_restart(".", "RESTART")

if __name__ == '__main__':
    wttest.run()



"""
Holder for a set of TestGroup instances.
"""

from __future__ import absolute_import

import time

from . import summary as _summary
from . import testgroup
from .. import config as _config
from .. import selector as _selector


class Suite(object):
    """
    A suite of tests.
    """

    TESTS_ORDER = ("cpp_unit_test", "cpp_integration_test", "db_test", "js_test", "mongos_test")

    def __init__(self, suite_name, suite_config):
        """
        Initializes the suite with the specified name and configuration.
        """

        self._suite_name = suite_name
        self._suite_config = suite_config

        self.test_groups = []
        for test_kind in Suite.TESTS_ORDER:
            if test_kind not in suite_config["selector"]:
                continue
            tests = self._get_tests_for_group(test_kind)
            test_group = testgroup.TestGroup(test_kind, tests)
            self.test_groups.append(test_group)

        self.return_code = None

        self._start_time = None
        self._end_time = None

    def _get_tests_for_group(self, test_kind):
        """
        Returns the tests to run based on the 'test_kind'-specific
        filtering policy.
        """

        test_info = self.get_selector_config()[test_kind]

        # The mongos_test doesn't have to filter anything, the test_info is just the arguments to
        # the mongos program to be used as the test case.
        if test_kind == "mongos_test":
            mongos_options = test_info  # Just for easier reading.
            if not isinstance(mongos_options, dict):
                raise TypeError("Expected dictionary of arguments to mongos")
            return [mongos_options]
        elif test_kind == "cpp_integration_test":
            tests = _selector.filter_cpp_integration_tests(**test_info)
        elif test_kind == "cpp_unit_test":
            tests = _selector.filter_cpp_unit_tests(**test_info)
        elif test_kind == "db_test":
            tests = _selector.filter_dbtests(**test_info)
        else:  # test_kind == "js_test":
            tests = _selector.filter_jstests(**test_info)

        if _config.ORDER_TESTS_BY_NAME:
            return sorted(tests, key=str.lower)
        return tests

    def get_name(self):
        """
        Returns the name of the test suite.
        """
        return self._suite_name

    def get_selector_config(self):
        """
        Returns the "selector" section of the YAML configuration.
        """
        return self._suite_config["selector"]

    def get_executor_config(self):
        """
        Returns the "executor" section of the YAML configuration.
        """
        return self._suite_config["executor"]

    def record_start(self):
        """
        Records the start time of the suite.
        """
        self._start_time = time.time()

    def record_end(self):
        """
        Records the end time of the suite.

        Sets the 'return_code' of the suite based on the record codes of
        each of the individual test groups.
        """

        self._end_time = time.time()

        # Only set 'return_code' if it hasn't been set already. It may have been set if there was
        # an exception that happened during the execution of the suite.
        if self.return_code is None:
            # The return code of the suite should be 2 if any test group has a return code of 2.
            # The return code of the suite should be 1 if any test group has a return code of 1,
            # and none have a return code of 2. Otherwise, the return code should be 0.
            self.return_code = max(test_group.return_code for test_group in self.test_groups)

    def summarize(self, sb):
        """
        Appends a summary of each individual test group onto the string
        builder 'sb'.
        """

        combined_summary = _summary.Summary(0, 0.0, 0, 0, 0, 0)

        summarized_groups = []
        for group in self.test_groups:
            group_sb = []
            summary = group.summarize(group_sb)
            summarized_groups.append("    %ss: %s" % (group.test_kind, "\n        ".join(group_sb)))

            combined_summary = _summary.combine(combined_summary, summary)

        if combined_summary.num_run == 0:
            sb.append("Suite did not run any tests.")
            return

        # Override the 'time_taken' attribute of the summary if we have more accurate timing
        # information available.
        if self._start_time is not None and self._end_time is not None:
            time_taken = self._end_time - self._start_time
            combined_summary = combined_summary._replace(time_taken=time_taken)

        sb.append("%d test(s) ran in %0.2f seconds"
                  " (%d succeeded, %d were skipped, %d failed, %d errored)" % combined_summary)

        for summary_text in summarized_groups:
            sb.append(summary_text)



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#

import re, os, sys
from distutils.core import setup, Extension

# OS X hack: turn off the Universal binary support that is built into the
# Python build machinery, just build for the default CPU architecture.
if not 'ARCHFLAGS' in os.environ:
    os.environ['ARCHFLAGS'] = ''

# Suppress warnings building SWIG generated code
extra_cflags = [ '-w', '-I../../src/include']

dir = os.path.dirname(__file__)

# Read the version information from the RELEASE_INFO file
for l in open(os.path.join(dir, '..', '..', 'RELEASE_INFO')):
    if re.match(r'WIREDTIGER_VERSION_(?:MAJOR|MINOR|PATCH)=', l):
        exec(l)

wt_ver = '%d.%d' % (WIREDTIGER_VERSION_MAJOR, WIREDTIGER_VERSION_MINOR)

setup(name='wiredtiger', version=wt_ver,
    ext_modules=[Extension('_wiredtiger',
                [os.path.join(dir, 'wiredtiger_wrap.c')],
        libraries=['wiredtiger'],
        extra_compile_args=extra_cflags,
    )],
    package_dir={'' : dir},
    packages=['wiredtiger'],
)



#!/usr/bin/python

#    Copyright 2012 10gen Inc.
#
#    This program is free software: you can redistribute it and/or  modify
#    it under the terms of the GNU Affero General Public License, version 3,
#    as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    As a special exception, the copyright holders give permission to link the
#    code of portions of this program with the OpenSSL library under certain
#    conditions as described in each individual source file and distribute
#    linked combinations including the program with the OpenSSL library. You
#    must comply with the GNU Affero General Public License in all respects
#    for all of the code used other than as permitted herein. If you modify
#    file(s) with this exception, you may extend this exception to your
#    version of the file(s), but you are not obligated to do so. If you do not
#    wish to do so, delete this exception statement from your version. If you
#    delete this exception statement from all source files in the program,
#    then also delete it in the license file.

"""Generate error_codes.{h,cpp} from error_codes.err.

Format of error_codes.err:

error_code("symbol1", code1)
error_code("symbol2", code2)
...
error_class("class1", ["symbol1", "symbol2, ..."])

Usage:
    python generate_error_codes.py <cpp|js> <path to error_codes.err> [options]
"""

usage_msg = "usage: %prog <cpp|js> /path/to/error_codes.err [options]"

from optparse import OptionParser
import sys
def main(argv):
    generator = argv[1]
    error_codes, error_classes = parse_error_definitions_from_file(argv[2])
    check_for_conflicts(error_codes, error_classes)
    if (generator == 'cpp'):
        if (len(argv) != 5):
            usage('Wrong number of arguments')
        cpp_gen = cpp_generator(error_codes, error_classes)
        cpp_gen.generate()
    elif (generator == 'js'):
        if (len(argv) != 4):
            usage('Wrong number of arguments')
        js_gen = js_generator(error_codes, error_classes)
        js_gen.generate()
    else:
        usage('Must specify which generator(s) to use.')

def die(message=None):
    sys.stderr.write(message or "Fatal error\n")
    sys.exit(1)

def usage(message=None):
    sys.stderr.write(__doc__)
    die(message)

def parse_error_definitions_from_file(errors_filename):
    errors_file = open(errors_filename, 'r')
    errors_code = compile(errors_file.read(), errors_filename, 'exec')
    error_codes = []
    error_classes = []
    eval(errors_code, dict(error_code=lambda *args: error_codes.append(args),
                           error_class=lambda *args: error_classes.append(args)))
    error_codes.sort(key=lambda x: x[1])
    return error_codes, error_classes

def check_for_conflicts(error_codes, error_classes):
    failed = has_duplicate_error_codes(error_codes)
    if has_duplicate_error_classes(error_classes):
        failed = True
    if has_missing_error_codes(error_codes, error_classes):
        failed = True
    if failed:
        die()

def has_duplicate_error_codes(error_codes):
    sorted_by_name = sorted(error_codes, key=lambda x: x[0])
    sorted_by_code = sorted(error_codes, key=lambda x: x[1])

    failed = False
    prev_name, prev_code = sorted_by_name[0]
    for name, code in sorted_by_name[1:]:
        if name == prev_name:
            sys.stdout.write('Duplicate name %s with codes %s and %s\n' % (name, code, prev_code))
            failed = True
        prev_name, prev_code = name, code

    prev_name, prev_code = sorted_by_code[0]
    for name, code in sorted_by_code[1:]:
        if code == prev_code:
            sys.stdout.write('Duplicate code %s with names %s and %s\n' % (code, name, prev_name))
            failed = True
        prev_name, prev_code = name, code

    return failed

def has_duplicate_error_classes(error_classes):
    names = sorted(ec[0] for ec in error_classes)

    failed = False
    prev_name = names[0]
    for name in names[1:]:
        if prev_name == name:
            sys.stdout.write('Duplicate error class name %s\n' % name)
            failed = True
        prev_name = name
    return failed

def has_missing_error_codes(error_codes, error_classes):
    code_names = set(ec[0] for ec in error_codes)
    failed = False
    for class_name, class_code_names in error_classes:
        for name in class_code_names:
            if name not in code_names:
                sys.stdout.write('Undeclared error code %s in class %s\n' % (name, class_name))
                failed = True
    return failed

class base_generator(object):
    def __init__(self, error_codes, error_classes):
        self.error_codes = error_codes
        self.error_classes = error_classes

    def parseOptions(self, options, usage_msg):
        parser = OptionParser(usage=usage_msg)
        for (f,d,n,m,h) in options:
            parser.add_option(f,dest=d,nargs=n,metavar=m,help=h)
        (options, args) = parser.parse_args()
        return options


class js_generator(base_generator):
    def __init__(self, error_codes, error_classes):
        super(js_generator, self).__init__(error_codes, error_classes)
        options = [('--js-source','js_source',1,'DEST_JS_SOURCE','specify dest JS source file to save to')]
        options = self.parseOptions(options, usage_msg)
        if (options.js_source):
            self.js_source = options.js_source
        else:
            usage('Must specify JS source files')

    def generate(self):
        self.generate_source()

    def generate_source(self):
        string_to_int_cases = ',\n    '.join(
            '%s: %s' % (ec[0], ec[1]) for ec in self.error_codes)
        int_to_string_cases = ',\n    '.join(
            '%s: \'%s\'' % (ec[1], ec[0]) for ec in self.error_codes)
        predicate_definitions = '\n\n'.join(
            self.generate_error_class_predicate_definition(*ec) for ec in self.error_classes)
        open(self.js_source, 'wb').write(self.source_template % dict(
                string_to_int_cases=string_to_int_cases,
                int_to_string_cases=int_to_string_cases
                ))

    def generate_error_class_predicate_definition(self, class_name, code_names):
        cases = '\n        '.join('case \'%s\':' % c for c in code_names)
        return self.error_class_predicate_template % dict(class_name=class_name, cases=cases)

    source_template = '''// AUTO-GENERATED FILE DO NOT EDIT
// See src/mongo/base/generate_error_codes.py
/*    Copyright 2015 MongoDB, Inc.
*
*    This program is free software: you can redistribute it and/or  modify
*    it under the terms of the GNU Affero General Public License, version 3,
*    as published by the Free Software Foundation.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU Affero General Public License for more details.
*
*    You should have received a copy of the GNU Affero General Public License
*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*    As a special exception, the copyright holders give permission to link the
*    code of portions of this program with the OpenSSL library under certain
*    conditions as described in each individual source file and distribute
*    linked combinations including the program with the OpenSSL library. You
*    must comply with the GNU Affero General Public License in all respects
*    for all of the code used other than as permitted herein. If you modify
*    file(s) with this exception, you may extend this exception to your
*    version of the file(s), but you are not obligated to do so. If you do not
*    wish to do so, delete this exception statement from your version. If you
*    delete this exception statement from all source files in the program,
*    then also delete it in the license file.
*/

var ErrorCodes = {
    %(string_to_int_cases)s
};

var ErrorCodeStrings = {
    %(int_to_string_cases)s
};
'''

    error_class_predicate_template = '''function is%(class_name)s(err) {
    if (typeof err === 'string') {
        error = err;
    } else if (typeof err === 'number') {
        error = ErrorCodeStrings[err];
    }
    switch (error) {
        %(cases)s
            return true;
        default:
            return false;
    }
}
'''

class cpp_generator(base_generator):
    def __init__(self, error_codes, error_classes):
        super(cpp_generator, self).__init__(error_codes, error_classes)
        options = [('--cpp-header','cpp_header',1,'DEST_CPP_HEADER','specify dest CPP header file to save to'), ('--cpp-source','cpp_source',1,'DEST_CPP_SOURCE','specify dest CPP source file to save to')]
        options = self.parseOptions(options, usage_msg)
        if (options.cpp_header and options.cpp_source):
            self.cpp_header = options.cpp_header
            self.cpp_source = options.cpp_source
        else:
            usage('Must specify CPP header and source files')

    def generate(self):
        self.generate_header()
        self.generate_source()

    def generate_header(self):

        enum_declarations = ',\n            '.join('%s = %s' % ec for ec in self.error_codes)
        predicate_declarations = ';\n        '.join(
            'static bool is%s(Error err)' % ec[0] for ec in self.error_classes)

        open(self.cpp_header, 'wb').write(self.header_template % dict(
                error_code_enum_declarations=enum_declarations,
                error_code_class_predicate_declarations=predicate_declarations))

    def generate_source(self):
        symbol_to_string_cases = ';\n        '.join(
            'case %s: return "%s"' % (ec[0], ec[0]) for ec in self.error_codes)
        string_to_symbol_cases = ';\n        '.join(
            'if (name == "%s") return %s' % (ec[0], ec[0])
            for ec in self.error_codes)
        int_to_symbol_cases = ';\n        '.join(
            'case %s: return %s' % (ec[0], ec[0]) for ec in self.error_codes)
        predicate_definitions = '\n    '.join(
            self.generate_error_class_predicate_definition(*ec) for ec in self.error_classes)
        open(self.cpp_source, 'wb').write(self.source_template % dict(
                symbol_to_string_cases=symbol_to_string_cases,
                string_to_symbol_cases=string_to_symbol_cases,
                int_to_symbol_cases=int_to_symbol_cases,
                error_code_class_predicate_definitions=predicate_definitions))

    def generate_error_class_predicate_definition(self, class_name, code_names):
        cases = '\n        '.join('case %s:' % c for c in code_names)
        return self.error_class_predicate_template % dict(class_name=class_name, cases=cases)

    header_template = '''// AUTO-GENERATED FILE DO NOT EDIT
// See src/mongo/base/generate_error_codes.py
/*    Copyright 2014 MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or  modify
 *    it under the terms of the GNU Affero General Public License, version 3,
 *    as published by the Free Software Foundation.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the GNU Affero General Public License in all respects
 *    for all of the code used other than as permitted herein. If you modify
 *    file(s) with this exception, you may extend this exception to your
 *    version of the file(s), but you are not obligated to do so. If you do not
 *    wish to do so, delete this exception statement from your version. If you
 *    delete this exception statement from all source files in the program,
 *    then also delete it in the license file.
 */
#pragma once
#include <string>
#include "mongo/base/string_data.h"
namespace mongo {
    /**
     * This is a generated class containing a table of error codes and their corresponding error
     * strings. The class is derived from the definitions in src/mongo/base/error_codes.err file.
     *
     * Do not update this file directly. Update src/mongo/base/error_codes.err instead.
     */
    class ErrorCodes {
    public:
        enum Error {
            %(error_code_enum_declarations)s,
            MaxError
        };
        static std::string errorString(Error err);
        /**
         * Parses an Error from its "name".  Returns UnknownError if "name" is unrecognized.
         *
         * NOTE: Also returns UnknownError for the string "UnknownError".
         */
        static Error fromString(StringData name);
        /**
         * Casts an integer "code" to an Error.  Unrecognized codes are preserved, meaning
         * that the result of a call to fromInt() may not be one of the values in the
         * Error enumeration.
         */
        static Error fromInt(int code);
        %(error_code_class_predicate_declarations)s;
    };
}  // namespace mongo
'''

    source_template = '''// AUTO-GENERATED FILE DO NOT EDIT
// See src/mongo/base/generate_error_codes.py
/*    Copyright 2014 MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or  modify
 *    it under the terms of the GNU Affero General Public License, version 3,
 *    as published by the Free Software Foundation.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the GNU Affero General Public License in all respects
 *    for all of the code used other than as permitted herein. If you modify
 *    file(s) with this exception, you may extend this exception to your
 *    version of the file(s), but you are not obligated to do so. If you do not
 *    wish to do so, delete this exception statement from your version. If you
 *    delete this exception statement from all source files in the program,
 *    then also delete it in the license file.
 */
#include "mongo/base/error_codes.h"
#include "mongo/util/mongoutils/str.h"
namespace mongo {
    std::string ErrorCodes::errorString(Error err) {
        switch (err) {
        %(symbol_to_string_cases)s;
        default: return mongoutils::str::stream() << "Location" << err;
        }
    }
    ErrorCodes::Error ErrorCodes::fromString(StringData name) {
        %(string_to_symbol_cases)s;
        return UnknownError;
    }
    ErrorCodes::Error ErrorCodes::fromInt(int code) {
        return static_cast<Error>(code);
    }
    %(error_code_class_predicate_definitions)s
namespace {
    static_assert(sizeof(ErrorCodes::Error) == sizeof(int), "sizeof(ErrorCodes::Error) == sizeof(int)");
}  // namespace
}  // namespace mongo
'''

    error_class_predicate_template = '''bool ErrorCodes::is%(class_name)s(Error err) {
        switch (err) {
        %(cases)s
            return true;
        default:
            return false;
        }
    }
'''


if __name__ == '__main__':
    main(sys.argv)



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
# WiredTiger variable-length packing and unpacking functions

"""Packing and unpacking functions
The format string uses the following conversions:
Format  Python  Notes
  x     N/A     pad byte, no associated value
  b     int     signed byte
  B     int     unsigned byte
  h     int     signed 16-bit
  H     int     unsigned 16-bit
  i     int     signed 32-bit
  I     int     unsigned 32-bit
  l     int     signed 32-bit
  L     int     unsigned 32-bit
  q     int     signed 64-bit
  Q     int     unsigned 64-bit
  r     int     record number
  s     str     fixed-length string
  S     str     NUL-terminated string
  t     int     fixed-length bit field
  u     str     raw byte array
"""

from intpacking import pack_int, unpack_int

def __get_type(fmt):
    if not fmt:
        return None, fmt
    # Variable-sized encoding is the default (and only supported format in v1)
    if fmt[0] in '.@<>':
        tfmt = fmt[0]
        fmt = fmt[1:]
    else:
        tfmt = '.'
    return tfmt, fmt

def __unpack_iter_fmt(fmt):
    size = 0
    havesize = 0
    for offset, char in enumerate(fmt):
        if char.isdigit():
            size = (size * 10) + int(char)
            havesize = 1
        else:
            if not havesize:
                size = 1
            yield offset, havesize, size, char
            size = 0
            havesize = 0

def unpack(fmt, s):
    tfmt, fmt = __get_type(fmt)
    if not fmt:
        return ()
    if tfmt != '.':
        raise ValueError('Only variable-length encoding is currently supported')
    result = []
    for offset, havesize, size, f in __unpack_iter_fmt(fmt):
        if f == 'x':
            s = s[size:]
            # Note: no value, don't increment i
        elif f in 'SsUu':
            if not havesize:
                if f == 's':
                    pass
                elif f == 'S':
                    size = s.find('\0')
                elif f == 'u' and offset == len(fmt) - 1:
                    size = len(s)
                else:
                    # Note: 'U' is used internally, and may be exposed to us.
                    # It indicates that the size is always stored unless there
                    # is a size in the format.
                    size, s = unpack_int(s)
            result.append(s[:size])
            if f == 'S' and not havesize:
                size += 1
            s = s[size:]
        elif f in 't':
            # bit type, size is number of bits
            result.append(ord(s[0:1]))
            s = s[1:]
        elif f in 'Bb':
            # byte type
            for i in xrange(size):
                v = ord(s[0:1])
                if f != 'B':
                    v -= 0x80
                result.append(v)
                s = s[1:]
        else:
            # integral type
            for j in xrange(size):
                v, s = unpack_int(s)
                result.append(v)
    return result

def __pack_iter_fmt(fmt, values):
    index = 0
    for offset, havesize, size, char in __unpack_iter_fmt(fmt):
        if char == 'x':  # padding no value
            yield offset, havesize, size, char, None
        elif char in 'SsUut':
            yield offset, havesize, size, char, values[index]
            index += 1
        else:            # integral type
            size = size if havesize else 1
            for i in xrange(size):
                value = values[index]
                yield offset, havesize, 1, char, value
                index = index + 1

def pack(fmt, *values):
    tfmt, fmt = __get_type(fmt)
    if not fmt:
        return ()
    if tfmt != '.':
        raise ValueError('Only variable-length encoding is currently supported')
    result = ''
    i = 0
    for offset, havesize, size, f, val in __pack_iter_fmt(fmt, values):
        if f == 'x':
            if not havesize:
                result += '\0'
            else:
                result += '\0' * size
            # Note: no value, don't increment i
        elif f in 'SsUu':
            if f == 'S' and '\0' in val:
                l = val.find('\0')
            else:
                l = len(val)
            if havesize or f == 's':
                if l > size:
                    l = size
            elif (f == 'u' and offset != len(fmt) - 1) or f == 'U':
                result += pack_int(l)
            if type(val) is unicode and f in 'Ss':
                result += str(val[:l])
            else:
                result += val[:l]
            if f == 'S' and not havesize:
                result += '\0'
            elif size > l:
                result += '\0' * (size - l)
        elif f in 't':
            # bit type, size is number of bits
            if not havesize:
                size = 1
            if size > 8:
                raise ValueError("bit count cannot be greater than 8 for 't' encoding")
            mask = (1 << size) - 1
            if (mask & val) != val:
                raise ValueError("value out of range for 't' encoding")
            result += chr(val)
        elif f in 'Bb':
            # byte type
            if not havesize:
                size = 1
            for i in xrange(size):
                if f == 'B':
                    v = val
                else:
                    # Translate to maintain ordering with the sign bit.
                    v = val + 0x80
                if v > 255 or v < 0:
                    raise ValueError("value out of range for 'B' encoding")
                result += chr(v)
        else:
            # integral type
            result += pack_int(val)
    return result



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import sys, threading, wiredtiger, wttest
from suite_subprocess import suite_subprocess
from wiredtiger import WiredTigerError
from wtscenario import make_scenarios

class Callback(wiredtiger.AsyncCallback):
    def __init__(self, current):
        self.current = current
        self.ncompact = 0
        self.ninsert = 0
        self.nremove = 0
        self.nsearch = 0
        self.nupdate = 0
        self.nerror = 0
        self.lock = threading.RLock()

    def notify_error(self, key, value, optype, exp, desc):
        self.tty('ERROR: notify(' + str(key) + ',' + str(value) + ',' +
            str(optype) + '): ' + 'Expected: ' + str(exp) + ' ' + desc)

    def notify(self, op, op_ret, flags):

        # Note: we are careful not to throw any errors here.  Any
        # exceptions would be swallowed by a non-python worker thread.
        try:
            optype = op.get_type()
            if optype != wiredtiger.WT_AOP_COMPACT:
                key = op.get_key()
                #
                # Remove does not set a value.  Just set it from the
                # reference list.  If an error is expected, the
                # reference list should list it as the expected value.
                #
                if op_ret != 0:
                    value = op_ret
                elif optype != wiredtiger.WT_AOP_REMOVE:
                    value = op.get_value()
                else:
                    value = self.current[key]

            if optype == wiredtiger.WT_AOP_INSERT:
                self.lock.acquire()
                self.ninsert += 1
                self.lock.release()
            elif optype == wiredtiger.WT_AOP_COMPACT:
                self.lock.acquire()
                self.ncompact += 1
                self.lock.release()
                # Skip checking key/value.
                return 0
            elif optype == wiredtiger.WT_AOP_REMOVE:
                self.lock.acquire()
                self.nremove += 1
                self.lock.release()
            elif optype == wiredtiger.WT_AOP_SEARCH:
                self.lock.acquire()
                self.nsearch += 1
                self.lock.release()
            elif optype == wiredtiger.WT_AOP_UPDATE:
                self.lock.acquire()
                self.nupdate += 1
                self.lock.release()
            else:
                self.notify_error(key, value, optype, 0, 'unexpected optype')
                self.lock.acquire()
                self.nerror += 1
                self.lock.release()
            if self.current[key] != value:
                self.notify_error(key, value, optype, self.current[key], 'unexpected value')
                self.lock.acquire()
                self.nerror += 1
                self.lock.release()
        except (BaseException) as err:
            self.tty('ERROR: exception in notify: ' + str(err))
            raise

        return 0

# test_async02.py
#    Async operations
# Basic smoke-test of file and table async ops: tests get/set key, insert
# update, and remove.
class test_async02(wttest.WiredTigerTestCase, suite_subprocess):
    """
    Test basic operations
    """
    table_name1 = 'test_async02'
    nentries = 100
    async_ops = nentries / 2
    async_threads = 3
    current = {}

    scenarios = make_scenarios([
        ('file-col', dict(tablekind='col',uri='file')),
        ('file-fix', dict(tablekind='fix',uri='file')),
        ('file-row', dict(tablekind='row',uri='file')),
        ('lsm-row', dict(tablekind='row',uri='lsm')),
        ('table-col', dict(tablekind='col',uri='table')),
        ('table-fix', dict(tablekind='fix',uri='table')),
        ('table-row', dict(tablekind='row',uri='table')),
    ])

    # Enable async for this test.
    def conn_config(self, dir):
        return 'async=(enabled=true,ops_max=%s,' % self.async_ops + \
            'threads=%s)' % self.async_threads

    def genkey(self, i):
        if self.tablekind == 'row':
            return 'key' + str(i)
        else:
            return long(i+1)

    def genvalue(self, i):
        if self.tablekind == 'fix':
            return int(i & 0xff)
        else:
            return 'value' + str(i)

    # Create and populate the object.
    def create_session(self, tablearg):
        if self.tablekind == 'row':
            keyformat = 'key_format=S'
        else:
            keyformat = 'key_format=r'  # record format
        if self.tablekind == 'fix':
            valformat = 'value_format=8t'
        else:
            valformat = 'value_format=S'
        create_args = keyformat + ',' + valformat

        self.pr('creating session: ' + create_args)
        self.session.create(tablearg, create_args)

    def test_ops(self):
        tablearg = self.uri + ':' + self.table_name1
        self.create_session(tablearg)

        # Populate our reference table first, so we don't need to
        # use locks to reference it.
        self.current = {}
        for i in range(0, self.nentries):
            k = self.genkey(i)
            v = self.genvalue(i)
            self.current[k] = v

        # Populate table with async inserts, callback checks
        # to ensure key/value is correct.
        callback = Callback(self.current)
        for i in range(0, self.nentries):
            self.pr('creating async op')
            op = self.conn.async_new_op(tablearg, None, callback)
            k = self.genkey(i)
            v = self.genvalue(i)
            op.set_key(k)
            op.set_value(v)
            op.insert()

        self.conn.async_flush()
        self.pr('flushed')

        k = self.genkey(self.nentries / 2)
        v = self.genvalue(self.nentries / 2)
        k1 = self.genkey(self.nentries + 1)
        v1 = self.genvalue(self.nentries + 1)
        self.current[k] = wiredtiger.WT_DUPLICATE_KEY
        self.current[k1] = wiredtiger.WT_NOTFOUND
        #
        # Error cases:
        # Check inserting an existing record - WT_DUPLICATE_KEY.
        # Check updating a non-existent record - WT_NOTFOUND.
        # Check removing a non-existent record - WT_NOTFOUND.
        #
        op = self.conn.async_new_op(tablearg, 'overwrite=false', callback)
        op.set_key(k)
        op.set_value(v)
        op.insert()
        op = self.conn.async_new_op(tablearg, 'overwrite=false', callback)
        op.set_key(k1)
        op.set_value(v1)
        op.update()
        op = self.conn.async_new_op(tablearg, 'overwrite=false', callback)
        op.set_key(k1)
        op.remove()

        # Wait for all outstanding async ops to finish.
        self.conn.async_flush()
        self.pr('flushed')

        # Make sure all callbacks went according to plan.
        self.assertTrue(callback.nerror == 0)

if __name__ == '__main__':
    wttest.run()



#!/usr/bin/env python

# This program pulls the function names from wiredtiger.in and generates
# an input file for Java SWIG that adds doxygen copydoc comments to functions.

import os, re, sys
import api_data
from dist import compare_srcfile

# Temporary file.
tmp_file = '__tmp'

#####################################################################
# Update wiredtiger.in with doxygen comments
#####################################################################
f='../src/include/wiredtiger.in'
o='../lang/java/java_doc.i'
tfile = open(tmp_file, 'w')

tfile.write('''/* DO NOT EDIT: automatically built by dist/java_doc.py. */

''')

cclass_re = re.compile('^struct __([a-z_]*) {')
cfunc_re = re.compile('\t.*? __F\(([a-z_]*)\)')

curr_class = ""
for line in open(f, 'r'):

    m = cclass_re.match(line)
    if m:
        curr_class = m.group(1)

    if curr_class == "":
        continue

    m = cfunc_re.match(line)
    if m:
        tfile.write('COPYDOC(__' + curr_class.lower() + ', ' +
        curr_class.upper() + ', ' + m.group(1) + ')\n')

tfile.close()
compare_srcfile(tmp_file, o)



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import os
import wiredtiger, wttest, run
from wtscenario import make_scenarios

# test_join06.py
#    Join operations
# Joins with a read-uncommitted
class test_join06(wttest.WiredTigerTestCase):
    nentries = 1000

    isoscen = [
        ('isolation_read_uncommitted', dict(isolation='read-uncommitted')),
        ('isolation_read_committed', dict(isolation='read-committed')),
        ('isolation_default', dict(isolation='')),
        ('isolation_snapshot', dict(isolation='snapshot'))
    ]

    bloomscen = [
        ('bloom', dict(bloom=True)),
        ('nobloom', dict(bloom=False))
    ]

    scenarios = make_scenarios(isoscen, bloomscen)

    def gen_values(self, i):
        s = str(i)                    # 345 => "345"
        f = s[0:1] + s[0:1] + s[0:1]  # 345 => "333"
        return [s, f]

    def gen_values2(self, i):
        s = str(i)                    # 345 => "345"
        l = s[-1:] + s[-1:] + s[-1:]  # 345 => "555"
        return [s, l]

    def populate(self, s, gen_values):
        c = s.open_cursor('table:join06', None, None)
        for i in range(0, self.nentries):
            c.set_key(i)
            c.set_value(*gen_values(i))
            c.insert()
        c.close()

    # Common function for testing the most basic functionality
    # of joins
    def test_join(self):
        self.session.create('table:join06',
                            'columns=(k,v0,v1),key_format=i,value_format=SS')
        self.session.create('index:join06:index0','columns=(v0)')
        self.session.create('index:join06:index1','columns=(v1)')

        self.populate(self.session, self.gen_values)

        # TODO: needed?
        #self.reopen_conn()

        if self.isolation != '':
            self.session.begin_transaction('isolation=' + self.isolation)

        jc = self.session.open_cursor('join:table:join06', None, None)
        c0 = self.session.open_cursor('index:join06:index0', None, None)
        c0.set_key('520')
        self.assertEquals(0, c0.search())
        self.session.join(jc, c0, 'compare=ge')

        joinconfig = 'compare=eq'
        if self.bloom:
            joinconfig += ',strategy=bloom,count=1000'
        c1 = self.session.open_cursor('index:join06:index1', None, None)
        c1.set_key('555')
        self.assertEquals(0, c1.search())
        self.session.join(jc, c1, joinconfig)

        if self.isolation == 'read-uncommitted' and self.bloom:
            # Make sure that read-uncommitted with Bloom is not allowed.
            # This is detected on the first next() operation.
            msg = '/cannot be used with read-uncommitted/'
            self.assertRaisesWithMessage(wiredtiger.WiredTigerError,
                lambda: jc.next(), msg)
            return

        # Changes made in another session may or may not be visible to us,
        # depending on the isolation level.
        if self.isolation == 'read-uncommitted':
            # isolation level is read-uncommitted, so we will see
            # additions deletions made in our other session.
            mbr = set(range(525,1000,10)) | set(range(55,100,10)) | set([520])
        else:
            # default isolation level, so we should see a consistent
            # set at the time we begin iteration.
            mbr = set(range(520,600)) | set(range(53,60))

        altered = False
        while jc.next() == 0:
            [k] = jc.get_keys()
            [v0,v1] = jc.get_values()
            #self.tty('GOT: ' + str(k) + ': ' + str(jc.get_values()))
            if altered and self.isolation == 'read-uncommitted':
                self.assertEquals(self.gen_values2(k), [v0, v1])
            else:
                self.assertEquals(self.gen_values(k), [v0, v1])
            if not k in mbr:
                self.tty('**** ERROR: result ' + str(k) + ' is not in: ' +
                         str(mbr))
            self.assertTrue(k in mbr)
            mbr.remove(k)

            # In another session, we remove entries for keys ending in 6,
            # and add entries for keys ending in 5.  Depending on the
            # isolation level for the transaction, these changes may or
            # may not be visible for the original session.
            if not altered:
                s = self.conn.open_session(None)
                s.begin_transaction(None)
                self.populate(s, self.gen_values2)
                s.commit_transaction()
                s.close()
                altered = True

        if len(mbr) != 0:
            self.tty('**** ERROR: did not see these: ' + str(mbr))
        self.assertEquals(0, len(mbr))

        jc.close()
        c1.close()
        c0.close()
        if self.isolation != '':
            self.session.commit_transaction()
        self.session.drop('table:join06')

if __name__ == '__main__':
    wttest.run()



"""
Customize the behavior of a fixture by allowing special code to be
executed before or after each test, and before or after each suite.
"""

from __future__ import absolute_import

import os
import sys

import bson
import pymongo

from . import fixtures
from . import testcases
from .. import errors
from .. import logging
from .. import utils


def make_custom_behavior(class_name, *args, **kwargs):
    """
    Factory function for creating CustomBehavior instances.
    """

    if class_name not in _CUSTOM_BEHAVIORS:
        raise ValueError("Unknown custom behavior class '%s'" % (class_name))
    return _CUSTOM_BEHAVIORS[class_name](*args, **kwargs)


class CustomBehavior(object):
    """
    The common interface all CustomBehaviors will inherit from.
    """

    @staticmethod
    def start_dynamic_test(hook_test_case, test_report):
        """
        If a CustomBehavior wants to add a test case that will show up
        in the test report, it should use this method to add it to the
        report, since we will need to count it as a dynamic test to get
        the stats in the summary information right.
        """
        test_report.startTest(hook_test_case, dynamic=True)

    def __init__(self, logger, fixture, description):
        """
        Initializes the CustomBehavior with the specified fixture.
        """

        if not isinstance(logger, logging.Logger):
            raise TypeError("logger must be a Logger instance")

        self.logger = logger
        self.fixture = fixture
        self.hook_test_case = None
        self.logger_name = self.__class__.__name__
        self.description = description


    def before_suite(self, test_report):
        """
        The test runner calls this exactly once before they start
        running the suite.
        """
        pass

    def after_suite(self, test_report):
        """
        The test runner calls this exactly once after all tests have
        finished executing. Be sure to reset the behavior back to its
        original state so that it can be run again.
        """
        pass

    def before_test(self, test, test_report):
        """
        Each test will call this before it executes.

        Raises a TestFailure if the test should be marked as a failure,
        or a ServerFailure if the fixture exits uncleanly or
        unexpectedly.
        """
        pass

    def after_test(self, test, test_report):
        """
        Each test will call this after it executes.

        Raises a TestFailure if the test should be marked as a failure,
        or a ServerFailure if the fixture exits uncleanly or
        unexpectedly.
        """
        pass


class CleanEveryN(CustomBehavior):
    """
    Restarts the fixture after it has ran 'n' tests.
    On mongod-related fixtures, this will clear the dbpath.
    """

    DEFAULT_N = 20

    def __init__(self, logger, fixture, n=DEFAULT_N):
        description = "CleanEveryN (restarts the fixture after running `n` tests)"
        CustomBehavior.__init__(self, logger, fixture, description)

        # Try to isolate what test triggers the leak by restarting the fixture each time.
        if "detect_leaks=1" in os.getenv("ASAN_OPTIONS", ""):
            self.logger.info("ASAN_OPTIONS environment variable set to detect leaks, so restarting"
                             " the fixture after each test instead of after every %d.", n)
            n = 1

        self.n = n
        self.tests_run = 0

    def after_test(self, test, test_report):
        self.tests_run += 1
        if self.tests_run >= self.n:
            self.logger.info("%d tests have been run against the fixture, stopping it...",
                             self.tests_run)
            self.tests_run = 0

            teardown_success = self.fixture.teardown()
            self.logger.info("Starting the fixture back up again...")
            self.fixture.setup()
            self.fixture.await_ready()

            # Raise this after calling setup in case --continueOnFailure was specified.
            if not teardown_success:
                raise errors.TestFailure("%s did not exit cleanly" % (self.fixture))


class JsCustomBehavior(CustomBehavior):
    def __init__(self, logger, fixture, js_filename, description, shell_options=None):
        CustomBehavior.__init__(self, logger, fixture, description)
        self.hook_test_case = testcases.JSTestCase(logger,
                                              js_filename,
                                              shell_options=shell_options,
                                              test_kind="Hook")

    def before_suite(self, test_report):
        # Configure the test case after the fixture has been set up.
        self.hook_test_case.configure(self.fixture)

    def after_test(self, test, test_report):
        description = "{0} after running '{1}'".format(self.description, test.short_name())
        try:
            # Change test_name and description to be more descriptive.
            self.hook_test_case.test_name = test.short_name() + ":" + self.logger_name
            CustomBehavior.start_dynamic_test(self.hook_test_case, test_report)
            self.hook_test_case.run_test()
            self.hook_test_case.return_code = 0
            test_report.addSuccess(self.hook_test_case)
        except self.hook_test_case.failureException as err:
            self.hook_test_case.logger.exception("{0} failed".format(description))
            test_report.addFailure(self.hook_test_case, sys.exc_info())
            raise errors.TestFailure(err.args[0])
        finally:
            test_report.stopTest(self.hook_test_case)


class ValidateCollections(JsCustomBehavior):
    """
    Runs full validation on all collections in all databases on every stand-alone
    node, primary replica-set node, or primary shard node.
    """

    DEFAULT_N = 1

    def __init__(self, logger, fixture, n=DEFAULT_N, improvedV33Validate=False):
        description = "Collection validation"
        js_filename = os.path.join("jstests", "hooks", "run_validate_collections.js")
        JsCustomBehavior.__init__(self, logger, fixture, js_filename, description)
        self.n = n
        self.tests_run = 0
        self.improved_validate = improvedV33Validate

    def after_test(self, test, test_report):
        self.tests_run += 1
        if self.tests_run >= self.n:
            self.tests_run = 0
            
            if not self.improved_validate:
                JsCustomBehavior.after_test(self, test, test_report)
                return
            elif not isinstance(self.fixture, fixtures.MongoDFixture):
                raise errors.TestFailure(("ValidateCollections with the improved validate command"
                                          " can only be run against a stand-alone mongod"))

            # Need to remember the dbpath and port of the original mongod.
            mongod_options = self.fixture.mongod_options.copy()
            mongod_options["queryableBackupMode"] = ""
            mongod_executable = "/data/multiversion/mongod-3.3.10"
            if sys.platform == "win32":
                mongod_executable += ".exe"
            
            self.fixture.reconfigure(mongod_executable, mongod_options)

            JsCustomBehavior.after_test(self, test, test_report)

            self.fixture.reset_configuration()


class CheckReplDBHash(JsCustomBehavior):
    """
    Checks that the dbhashes of all non-local databases and non-replicated system collections
    match on the primary and secondaries.
    """
    def __init__(self, logger, fixture):
        description = "Check dbhashes of all replica set or master/slave members"
        js_filename = os.path.join("jstests", "hooks", "run_check_repl_dbhash.js")
        JsCustomBehavior.__init__(self, logger, fixture, js_filename, description)


class TypeSensitiveSON(bson.SON):
    """
    Extends bson.SON to perform additional type-checking of document values
    to differentiate BSON types.
    """

    def items_with_types(self):
        """
        Returns a list of triples. Each triple consists of a field name, a
        field value, and a field type for each field in the document.
        """

        return [(key, self[key], type(self[key])) for key in self]

    def __eq__(self, other):
        """
        Comparison to another TypeSensitiveSON is order-sensitive and
        type-sensitive while comparison to a regular dictionary ignores order
        and type mismatches.
        """

        if isinstance(other, TypeSensitiveSON):
            return (len(self) == len(other) and
                    self.items_with_types() == other.items_with_types())

        raise TypeError("TypeSensitiveSON objects cannot be compared to other types")


_CUSTOM_BEHAVIORS = {
    "CleanEveryN": CleanEveryN,
    "CheckReplDBHash": CheckReplDBHash,
    "ValidateCollections": ValidateCollections,
}



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import os
import wiredtiger, wttest, run

# test_join04.py
#    Join operations
# Joins with a custom extractor, using equality joins
class test_join04(wttest.WiredTigerTestCase):
    table_name1 = 'test_join04'
    nentries = 100

    # Return the wiredtiger_open extension argument for a shared library.
    def extensionArg(self, exts):
        extfiles = []
        for ext in exts:
            (dirname, name, libname) = ext
            if name != None and name != 'none':
                testdir = os.path.dirname(__file__)
                extdir = os.path.join(run.wt_builddir, 'ext', dirname)
                extfile = os.path.join(
                    extdir, name, '.libs', 'libwiredtiger_' + libname + '.so')
                if not os.path.exists(extfile):
                    self.skipTest('extension "' + extfile + '" not built')
                if not extfile in extfiles:
                    extfiles.append(extfile)
        if len(extfiles) == 0:
            return ''
        else:
            return ',extensions=["' + '","'.join(extfiles) + '"]'

    # Override WiredTigerTestCase, we have extensions.
    def setUpConnectionOpen(self, dir):
        extarg = self.extensionArg([('extractors', 'csv', 'csv_extractor')])
        connarg = 'create,error_prefix="{0}: ",{1}'.format(
            self.shortid(), extarg)
        conn = self.wiredtiger_open(dir, connarg)
        self.pr(`conn`)
        return conn

    # JIRA WT-2308:
    # Test extractors with equality joins
    def test_join_extractor(self):
        self.session.create('table:join04',
                            'key_format=i,value_format=S,columns=(k,v)')
        self.session.create('index:join04:index1',
                       'key_format=i,extractor=csv,' +
                       'app_metadata={"format" : "i", "field" : "0"}')
        self.session.create('index:join04:index2',
                       'key_format=i,extractor=csv,' +
                       'app_metadata={"format" : "i", "field" : "1"}')

        cursor1 = self.session.open_cursor('table:join04', None, None)
        cursor1[1] = '10,21'
        cursor1[2] = '10,22'
        cursor1.close()

        cursor1 = self.session.open_cursor('index:join04:index1', None, None)
        cursor1.set_key(10)
        cursor1.search()
        cursor2 = self.session.open_cursor('index:join04:index2', None, None)
        cursor2.set_key(22)
        cursor2.search()

        jcursor = self.session.open_cursor('join:table:join04', None, None)
        self.session.join(jcursor, cursor1, 'compare=eq')
        self.session.join(jcursor, cursor2, 'compare=eq')

        found = 0
        while jcursor.next() == 0:
            [k] = jcursor.get_keys()
            [v] = jcursor.get_values()
            self.assertEqual(k, 2)
            self.assertEqual(v, '10,22')
            found += 1
        self.assertEqual(found, 1)
        jcursor.close()
        cursor1.close()
        cursor2.close()

    # More tests using extractors with equality joins
    def test_join_extractor_more(self):
        self.session.create('table:join04',
                            'key_format=i,value_format=S,columns=(k,v)')
        self.session.create('index:join04:index1',
                       'key_format=i,extractor=csv,' +
                       'app_metadata={"format" : "i", "field" : "0"}')
        self.session.create('index:join04:index2',
                       'key_format=i,extractor=csv,' +
                       'app_metadata={"format" : "i", "field" : "1"}')
        self.session.create('index:join04:index3',
                       'key_format=i,extractor=csv,' +
                       'app_metadata={"format" : "i", "field" : "2"}')

        jcursor = self.session.open_cursor('join:table:join04', None, None)
        cursor1 = self.session.open_cursor('table:join04', None, None)
        k = 1
        for v in ['10,21,30','10,22,30','10,23,30',
                  '11,21,30','11,22,30','11,23,30',
                  '10,21,31','10,22,31','10,23,31',
                  '10,21,30','11,22,31','12,23,32']:
            cursor1[k] = v
            k += 1
        cursor1.close()

        # A list of tests, one per line, each entry is:
        #    [[list of inputs], [list of outputs]]
        tests = [
            [[10,22,30], ['10,22,30']],
            [[10,21,30], ['10,21,30','10,21,30']],
            [[11], ['11,21,30','11,22,30','11,23,30','11,22,31']],
            [[None,22], ['10,22,30','11,22,30','10,22,31','11,22,31']]]

        for t in tests:
            jcursor = self.session.open_cursor('join:table:join04', None, None)
            ins = t[0]
            outs = t[1]
            cursors = []
            n = 0
            for k in ins:
                n += 1
                if k == None: continue
                uri = 'index:join04:index' + str(n)
                c = self.session.open_cursor(uri, None, None)
                c.set_key(k)
                self.assertEqual(c.search(), 0)
                cursors.append(c)
                self.session.join(jcursor, c, 'compare=eq')
            while jcursor.next() == 0:
                [k] = jcursor.get_keys()
                [v] = jcursor.get_values()
                #self.tty('got=' + str(v) + ' at key=' + str(k))
                self.assertTrue(v in outs)
                outs.remove(v)
            self.assertEqual(len(outs), 0)
            jcursor.close()
            for c in cursors:
                c.close()

if __name__ == '__main__':
    wttest.run()



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import wiredtiger, wttest
from helper import copy_wiredtiger_home, key_populate, simple_populate

# test_bug015.py
#    JIRA WT-2162: index drop in a certain order triggers NULL pointer deref
class test_bug015(wttest.WiredTigerTestCase):
    def test_bug015(self):
        table = 'table:test_bug015'
        idx1 = 'index:test_bug015:aab'
        idx2 = 'index:test_bug015:aaa'
        self.session.create(table, "columns=(k,v)")
        self.session.create(idx1, "columns=(v)")
        self.session.create(idx2, "columns=(v)")
        self.session.drop(idx1, "force=true")
        self.session.create(idx1, "columns=(v)")
        self.session.drop(idx2, "force=true")
        self.session.create(idx2, "columns=(v)")

if __name__ == '__main__':
    wttest.run()



#!/usr/bin/env python
#
# Public Domain 2014-2016 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import os, time
import wiredtiger, wttest
from helper import complex_populate, simple_populate
from wtscenario import make_scenarios

# test_rebalance.py
#    session level rebalance operation
class test_rebalance(wttest.WiredTigerTestCase):
    name = 'test_rebalance'

    # Use small pages so we generate some internal layout
    # Setup LSM so multiple chunks are present
    config = 'key_format=S,allocation_size=512,internal_page_max=512' + \
             ',leaf_page_max=1k,lsm=(chunk_size=512k,merge_min=10)'

    scenarios = make_scenarios([
        ('file', dict(uri='file:')),
        ('table', dict(uri='table:')),
        ('lsm', dict(uri='lsm:'))
    ])

    # Populate an object, then rebalance it.
    def rebalance(self, populate, with_cursor):
        uri = self.uri + self.name
        populate(self, uri, self.config, 10000)

        # Force to disk, we don't rebalance in-memory objects.
        self.reopen_conn()

        # Open cursors should cause failure.
        if with_cursor:
            cursor = self.session.open_cursor(uri, None, None)
            self.assertRaises(wiredtiger.WiredTigerError,
                lambda: self.session.rebalance(uri, None))
            cursor.close()

        self.session.rebalance(uri, None)
        self.session.drop(uri)

    # Test rebalance of an object.
    def test_rebalance(self):
        # Simple file or table object.
        self.rebalance(simple_populate, False)
        self.rebalance(simple_populate, True)

        # A complex, multi-file table object.
        if self.uri == "table:":
            self.rebalance(complex_populate, False)
            self.rebalance(complex_populate, True)

if __name__ == '__main__':
    wttest.run()
